<?php
namespace OCA\Dematpayslip\Controller;

use OC\Files\Filesystem;
use OCA\DAV\Db\Direct;
use OCA\Gallery\Environment\NotFoundEnvException;
use OCP\Files\NotFoundException;
use OCP\IConfig;
use OCP\IL10N;
use OCP\IRequest;
use OCP\AppFramework\Http\TemplateResponse;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\Http\JSONResponse;
use OCP\AppFramework\Controller;

use OC_App;
use OCP\Files\Folder;
use OCA\Dematpayslip\Db\DematpayslipArchive;
use OCA\Dematpayslip\Db\DematpayslipArchiveMapper;
use OCA\Dematpayslip\Db\DematpayslipProcess;
use OCA\Dematpayslip\Db\DematpayslipProcessMapper;
use OCA\Dematpayslip\Db\DematpayslipProcessSkip;
use OCA\Dematpayslip\Db\DematpayslipProcessSkipMapper;
use OCA\Dematpayslip\Db\DematpayslipUser;
use OCA\Dematpayslip\Db\DematpayslipUserMapper;

use OCA\Pastell\Db\PastellConfig;
use OCA\Pastell\Db\PastellConfigMapper;
use OCA\Pastell\Db\PastellConfigFlow;
use OCA\Pastell\Db\PastellConfigFlowMapper;

use OCA\Pastell\PastellAPI;
use phpseclib\Crypt\Hash;

class PageController extends Controller {
    const LOG_SUCCESS = 0;
    const LOG_DEBUG   = 1;
    const LOG_WARNING = 2;
    const LOG_ERROR   = 3;

    const PROCESS_TIME_UNLOCK = 600; // seconds (max execution time)

    const PHASE_ID_UPLOAD = 0;
    const PHASE_ID_CHECK  = 1;
    const PHASE_ID_INDEXATION = 2;

    const USER_REMOVE_UNKNOWN = 1; // remove unknown users from list (FIX SAML)

    /**
     * @var array Payslip PDF search field list
     */
    private $_payslipPDFSearchFieldList = array(
        'siret'     => array(3),
        'matricule' => array(10),
        'fullname'  => array(9),
        'zip_city'  => array(19, 20),
    );
//    private $_payslipPDFSearchFieldList = array(
//        'siret'     => array(4, 5),
//        'matricule' => array(20, 21),
//        'fullname'  => array(19, 20),
//        'zip_city'  => array(34, 35, 36, 37),
//    );

    /**
     * @var string CSV delimiter
     */
    private $_csvDelimiter = ';';

    /**
     * @var string CSV enclosure
     */
    private $_csvEnclosure = '"';

    /**
     * @var int CSV length
     */
    private $_csvLength = 0;

    /**
     * @var string CSV encode source
     */
    private $_csvEncodeFrom = 'UTF-8';

    /**
     * @var string CSV encode destination
     */
    private $_csvEncodeTo = 'UTF-8';

    /**
     * @var File handler Log file
     */
    private $_logFile = null;

    /**
     * @var int Log level
     */
    private $_logLevel = self::LOG_DEBUG;

    /**
     * @var bool Fail on first error
     */
    private $_processFailOnFirstError = true;

    /** @var IConfig */
    private $config;

    /** @var IL10N */
    private $l;

    /** @var string User Id */
    private $userId;

    /**@var array dematpayslip config */
    private $_dematpayslipConfigList = array();

    /** @var DematpayslipProcessMapper */
	private $_dematpayslipProcessMapper;
    private $_process;
    private $_canCancelProcess = false;

    /** @var Folder path */
    private $_uploadDirUserPath;
    private $_uploadDirFullPath;
    private $_checkDirUserPath;
    private $_checkDirFullPath;
    private $_reportDirUserPath;
    private $_reportDirFullPath;
    private $_exportDirUserPath;
    private $_exportDirFullPath;
    private $_tmpDirUserPath;
    private $_tmpDirFullPath;

    /** @var string payslip no index dir */
    private $_payslipNoIndexDir = 'BulletinsNoIndex';

    /** @var DematpayslipProcessSkipMapper */
    private $_dematpayslipProcessSkipMapper;

    /** @var DematpayslipUserMapper */
    private $_dematpayslipUserMapper;

    // messages
    private $_msgList = array(
        'error'   => array(),
        'success' => array(),
        'warning' => array(),
    );

    /** @var array DematpayslipUser */
    private $_dematpayslipUserList;

    /** @var array User */
    private $_userList;


    /**
     * Check if payslip file has not been archived yet
     *
     * @param   DematpayslipArchiveMapper   $dematpayslipArchiveMapper      Archive table mapper
     * @param   DematpayslipUser            $agentUser                      Agent user
     * @param   string                      $startDate                      Period start date
     * @param   string                      $endDate                        Period end date
     * @return  mixed|DematpayslipArchive|\OCP\AppFramework\Db\Entity|null
     *
     * @throws \Exception
     */
    private function _archiveCheckBeforeSave(DematpayslipArchiveMapper $dematpayslipArchiveMapper, DematpayslipUser $agentUser, $startDate, $endDate) {
        $dematpayslipArchive = null;
        $dematpayslipArchiveExistList = $dematpayslipArchiveMapper->findAllByAgentAndPeriod($agentUser->getSiret(), $agentUser->getMatricule(), $agentUser->getLastname(), $agentUser->getFirstname(), $startDate, $endDate);

        if (empty($dematpayslipArchiveExistList)) {
            $dematpayslipArchive = new DematpayslipArchive();
        } elseif (count($dematpayslipArchiveExistList) === 1) {
            if (!empty($dematpayslipArchiveExistList[0]->getPastellIdDocument())) {
                $warningMsg = $this->l->t('Le bulletin de paie de l\'agent "' . $agentUser->getLastname() . ' ' . $agentUser->getFirstname() . '" pour la période du ' . $startDate . ' au ' . $endDate . ' est déjà en cours d\'archivage.');
                $this->_logCsvLine($warningMsg, self::LOG_WARNING, $agentUser, $startDate, $endDate);
                $this->_processWarningMsg($warningMsg);
            } else {
                $dematpayslipArchive = $dematpayslipArchiveExistList[0];
            }
        } else {
            $warningMsg = $this->l->t('Plusieurs bulletins de paie de l\'agent "' . $agentUser->getLastname() . ' ' . $agentUser->getFirstname() . '" pour la période du ' . $startDate . ' au ' . $endDate . ' existent déjà.');
            $this->_logCsvLine($warningMsg, self::LOG_WARNING, $agentUser, $startDate, $endDate);
            $this->_processWarningMsg($warningMsg);
        }

        return $dematpayslipArchive;
    }

    /**
     * Save archive file for an agent and a period
     *
     * @param   DematpayslipArchiveMapper   $dematpayslipArchiveMapper  Archive table mapper
     * @param   DematpayslipArchive         $dematpayslipArchive        Archive
     * @param   string                      $archiveFilePath            Archive file path
     * @param   DematpayslipUser            $agentUser                  Agent user
     * @param   string                      $startDate                  Period start date
     * @param   string                      $endDate                    Period end date
     * @param   string                      $year                       Period year
     * @param   string                      $month                      Period month
     * @param   string                      $editionDate                Edition date
     * @param   int                         $importTime                 Import time
     *
     * @throws  \Exception
     */
    private function _archiveSaveForAgentAndPeriod(DematpayslipArchiveMapper $dematpayslipArchiveMapper, DematpayslipArchive $dematpayslipArchive, $archiveFilePath, DematpayslipUser $agentUser, $startDate, $endDate, $year, $month, $editionDate, $importTime) {
        $archiveFile         = Filesystem::getFileInfo($archiveFilePath);
        $archiveFileChecksum = Filesystem::hash('md5', $archiveFilePath);

        $dematpayslipArchive->setIdFile($archiveFile->getId());
        $dematpayslipArchive->setFilePath($archiveFile->getPath());
        $dematpayslipArchive->setIdUser($agentUser->getUserId());
        $dematpayslipArchive->setDematpayslipIdUser($agentUser->getId());
        $dematpayslipArchive->setSiret($agentUser->getSiret());
        $dematpayslipArchive->setMatricule($agentUser->getMatricule());
        $dematpayslipArchive->setYear($year);
        $dematpayslipArchive->setMonth($month);
        $dematpayslipArchive->setLastname($agentUser->getLastname());
        $dematpayslipArchive->setFirstname($agentUser->getFirstname());
        $dematpayslipArchive->setStartDate($startDate);
        $dematpayslipArchive->setEndDate($endDate);
        $dematpayslipArchive->setEditionDate($editionDate);
        $dematpayslipArchive->setImportTime($importTime);
        $dematpayslipArchive->setChecksum($archiveFileChecksum);

        if (empty($dematpayslipArchive->getId())) {
            // insert new file to archive
            $dematpayslipArchiveMapper->insert($dematpayslipArchive);
        } else {
            // update file to archive
            $dematpayslipArchiveMapper->update($dematpayslipArchive);
        }
    }

    /**
     * Assign params for template demat
     *
     * @return  array   Params for template
     */
    private function _assignParamsDemat() {
        return array(
            'canCancelProcess' => $this->_canCancelProcess,
            'msgList'          => $this->_msgList,
            'processArr'       => $this->_process->jsonSerialize(),
            'userArr'          => array('id' => $this->userId)
        );
    }

    /**
     * Assign params for agent list
     *
     * @return  array   Params for template
     */
    private function _assignAgentList() {
        return array(
            'dematpayslipUserList'  => $this->_dematpayslipUserMapper->findAll(),
            'exportDirUserPath'     => $this->_exportDirUserPath,
            'msgList'               => $this->_msgList,
        );
    }

    /**
     * Assign params for archive list
     *
     * @param   int         $archived               [=-1] for all, 0 for not archived and 1 for archived
     * @param   int         $refresh                [=false] not to refresh list form API, else true
     * @return  array       Params for template
     */
    private function _assignArchiveList($archived = -1, $refresh = false) {
        $paramList = array(
            'archived' => $archived
        );

        // get archive list
        $dematpayslipArchiveMapper = new DematpayslipArchiveMapper(\OC::$server->getDatabaseConnection());

        if ($refresh === true) {
            $dematpayslipArchiveList = $dematpayslipArchiveMapper->findAllArchived($archived);

            if (!empty($dematpayslipArchiveList)) {
                foreach ($dematpayslipArchiveList as $dematpayslipArchive) {
                    if (empty($dematpayslipArchive->getPastellIdDocument()))  continue;

                    $archiveFlowNum = 1; // archiving for 5 years by default

                    // get dematpayslip user
                    if (!empty($dematpayslipArchive->getDematpayslipIdUser())) {
                        $dematpayslipUser = $dematpayslipArchiveMapper->find($dematpayslipArchive->getDematpayslipIdUser());
                        if ($dematpayslipUser->getDisabled() == 0) {
                            $archiveFlowNum = 2; // archiving for 50 years
                        }
                    }

                    // Pastell API config
                    $pastellConfigFlowId = $this->config->getAppValue($this->appName, 'pastell_config_flow' . $archiveFlowNum . '_id', '');
                    $pastellConfigFlowMapper = new PastellConfigFlowMapper(\OC::$server->getDatabaseConnection());
                    $pastellConfigFlow = $pastellConfigFlowMapper->find($pastellConfigFlowId);

                    try {
                        // Pastell API connection
                        $pastellAPI = new PastellAPI($pastellConfigFlow->getIdConfig());

                        // get and modify document last action
                        $responseData = $pastellAPI->getEntityDocument($pastellConfigFlow->getIdE(), $dematpayslipArchive->getPastellIdDocument());
                        if (!empty($responseData['last_action']['action'])) {
                            $dematpayslipArchive->setPastellLastAction($responseData['last_action']['action']);
                            $dematpayslipArchiveMapper->update($dematpayslipArchive);
                        }
                    } catch (\Exception $e) {
                        $this->_msgList['error'][] = $e->getMessage();
                    }
                }
            }
        }

        $dematpayslipArchiveList = $dematpayslipArchiveMapper->findAllArchived($archived);

        return array(
            'dematpayslipArchiveList' => $dematpayslipArchiveList,
            'exportDirUserPath'       => $this->_exportDirUserPath,
            'msgList'                 => $this->_msgList,
            'paramList'               => $paramList
        );
    }

    /**
     * Assign params for user list
     *
     * @param   array   $searchList     Search values
     * @return  array   Params for template
     */
    private function _assignUserList($searchList) {
        return array(
            'msgList' => $this->_msgList,
            'searchList' => $searchList,
            'userList' => $this->_userList,
        );
    }

    /**
     * Assign params for template shares
     *
     * @return  array   Params for template
     */
    private function _assignParamsShares() {
        return array(
            'dematpayslipUserList' => $this->_dematpayslipUserList,
            'msgList'              => $this->_msgList,
        );
    }

    /**
     * Get phase name
     *
     * @param   int         $phaseId    Phase Id
     * @return  string      Phase name
     */
    private static function _getPhaseName($phaseId) {
        $phaseName = '';

        switch ($phaseId) {
            case self::PHASE_ID_UPLOAD :
                $phaseName = 'Upload';
            break;

            case self::PHASE_ID_CHECK :
                $phaseName = 'Vérification';
            break;

            case self::PHASE_ID_INDEXATION :
                $phaseName = 'Indexation';
            break;

            default : break;
        }

        return $phaseName;
    }

    /**
     * Find CSV and PDF files in uploaded directory
     *
     * @return  array   CSV and PDF filenames
     */
    private function _findUploadedCSVAndPDF() {
        $uploadFileCSVName = '';
        $uploadFilePDFName = '';
        $uploadDirFullPath = $this->_uploadDirFullPath;
        if (file_exists($uploadDirFullPath)) {
            $scanDir = scandir($uploadDirFullPath);
            if (!empty($scanDir)) {
                foreach ($scanDir as $fileName) {
                    if (!in_array($fileName, array('.', '..'))) {
                        $uploadFilePath = $uploadDirFullPath . DIRECTORY_SEPARATOR . $fileName;
                        if (is_file($uploadFilePath)) {
                            $uploadFileType = mime_content_type($uploadFilePath);
                            if ($uploadFileType == 'application/pdf') {
                                $uploadFilePDFName = $fileName;
                            } else if ($uploadFileType == 'text/plain') {
                                $uploadFileCSVName = $fileName;
                            }
                        }
                    }

                    if (!empty($uploadFileCSVName) && !empty($uploadFilePDFName)) {
                        break;
                    }
                }
            }
        }

        return array('csv' => $uploadFileCSVName, 'pdf' => $uploadFilePDFName);
    }

    /**
     * Find all files (recursive)
     *
     * @param   string      $filePath       File path
     * @return  array
     */
    private function _findAllFiles($filePath, &$fileList = array()) {
        $isDir = Filesystem::is_dir($filePath);

        if ($isDir) {
            $payslipPDFFileList = Filesystem::getDirectoryContent($filePath);

            foreach ($payslipPDFFileList as $payslipPDFFile) {
                $newFilePath = $filePath . DIRECTORY_SEPARATOR . $payslipPDFFile->getName();
                $isFile = Filesystem::is_file($newFilePath);
                if ($isFile) {
                    $fileList[] = $payslipPDFFile;
                } else {
                    $this->_findAllFiles($newFilePath, $fileList);
                }
            }
        }

        return $fileList;
    }

    /**
     * Remove all files in a directory
     *
     * @param   string      $dirPath    Directory path
     */
    private static function _removeAllFilesDir($dirPath) {
        $scanDir = scandir($dirPath);
        if (!empty($scanDir)) {
            foreach ($scanDir as $fileName) {
                if (!in_array($fileName, array('.', '..'))) {
                    $filePath = $dirPath . DIRECTORY_SEPARATOR . $fileName;
                    if (is_file($filePath)) {
                        unlink($filePath);
                    }
                }
            }
        }
    }

    /**
     * Remove all unknown users (FIX for multiple users with SAML)
     *
     * @param   array   $userFindByEmailList        User list
     * @return  array   Users list without unknown
     */
    private static function _removeUnknownUser($userFindByEmailList) {
        $userList = $userFindByEmailList;

        foreach ($userList as $key => $user) {
            if (preg_match('#^[0-9A-F]{8}\-[0-9A-F]{4}\-[0-9A-F]{4}\-[0-9A-F]{4}\-[0-9A-F]{12}#', $user->getDisplayName())) {
                unset($userList[$key]);
            }
        }

        return $userList;
    }

    /**
     * Cancel process
     */
    private function _processCancel() {
        if ($this->_process->getMustTerminate()<=0) {
            // vider le dossier Upload
            if (is_dir($this->_uploadDirFullPath)) {
                self::_removeAllFilesDir($this->_uploadDirFullPath);
            }

            // vider le dossier Check
            if (is_dir($this->_checkDirFullPath)) {
                self::_removeAllFilesDir($this->_checkDirFullPath);
            }

            $this->_processInit();
            $this->_dematpayslipProcessMapper->update($this->_process);
        }
    }

    /**
     * Set process error message
     *
     * @param   int         $errorCode      Error code
     * @param   string      $errorMsg       Error message
     * @param   string      $phaseStep      [=''] Phase step
     * @param   int         $phaseStepNum   [=0] Phase step num
     */
    private function _processErrorMsg($errorCode, $errorMsg, $phaseStep = '', $phaseStepNum = 0) {
        $this->_process->setHasError(1);
        $this->_msgList['error'][] = $errorMsg;
        $this->_process->setErrorCode($errorCode);
        $this->_process->setErrorMsg(implode('<br />', $this->_msgList['error']));
        $this->_process->setPhaseStep($phaseStep);
        $this->_process->setPhaseStepNum(intval($phaseStepNum));
        $this->_dematpayslipProcessMapper->update($this->_process);
    }

    /**
     * Set process warning message
     *
     * @param   string      $warningMsg       Warning message
     */
    private function _processWarningMsg($warningMsg) {
        $this->_msgList['warning'][] = $warningMsg;
    }

    /**
     * Csv log file
     *
     * @param   array   $colList    Column name list
     */
    private function _logCsvCols($colList) {
        if ($this->_logFile !== null) {
            fputcsv($this->_logFile, $colList, $this->_csvDelimiter, $this->_csvEnclosure);
        }
    }

    /**
     * Log csv line
     *
     * @param   string                  $msg            Log message
     * @param   int                     $level          [=LOG_DEBUG] Log level
     * @param   DematpayslipUser        $agentUser      [=null] Siret
     * @param   string                  $startDate      [=''] Start date
     * @param   string                  $endDate        [=''] End date
     *
     * @throws  \Exception
     */
    private function _logCsvLine($msg, $level = self::LOG_DEBUG, DematpayslipUser $agentUser = null, $startDate = '', $endDate = '') {
        if ($this->_logFile !== null) {
            if ($this->_logLevel <= $level || $level === self::LOG_SUCCESS) {
                $dateTimeNow = new \DateTime('now', \OC::$server->getDateTimeZone()->getTimeZone());
                $logLevel = '';
                switch ($level) {
                    case self::LOG_SUCCESS :
                        $logLevel = 'SUCCESS';
                        break;

                    case self::LOG_DEBUG :
                        $logLevel = 'DEBUG';
                        break;

                    case self::LOG_WARNING :
                        $logLevel = 'WARNING';
                        break;

                    case self::LOG_ERROR :
                        $logLevel = 'ERROR';
                        break;

                    default : break;
                }

                $csvLine = array(
                    $dateTimeNow->format('Y-m-d'),
                    $dateTimeNow->format('H:i:s'),
                    $logLevel,
                    $msg,
                    (!empty($agentUser) ? $agentUser->getSiret() : ''),
                    (!empty($agentUser) ? $agentUser->getMatricule() : ''),
                    (!empty($agentUser) ? $agentUser->getLastname() . ' ' . $agentUser->getFirstname() : ''),
                    (!empty($agentUser) ? $agentUser->getZipCity() : ''),
                    $startDate,
                    $endDate,
                    (!empty($agentUser) ? $agentUser->getEmail() : ''),
                );
                fputcsv($this->_logFile, $csvLine, $this->_csvDelimiter, $this->_csvEnclosure);
            }
        }
    }

    /**
     * Init process
     */
    private function _processInit() {
        $this->_process->setPhase(0);
        $this->_process->setPhaseStartTime(0);
        $this->_process->setPhaseStepNum(0);
        $this->_process->setMustTerminate(0);
        $this->_process->setHasError(0);
        $this->_process->setErrorCode(0);
        $this->_process->setErrorMsg('');
        $this->_process->setCsvOrigin('');
        $this->_process->setPdfOrigin('');
        $this->_process->setCsvCheck('');
        $this->_process->setUserId('');
    }

    /**
     * Check if process is locked (processing or locked by user)
     */
    private function _processCheck() {
        // deverrouillage automatique du process si trop de temps passe (ex : max execution time)
        if ((time() - $this->_process->getPhaseStartTime())>self::PROCESS_TIME_UNLOCK) {
            $this->_process->setProcessing(0);
            $this->_dematpayslipProcessMapper->update($this->_process);
        }

        $this->_canCancelProcess = false;
        // c'est le process lance par l'utilisteur
        if (!empty($this->_process->getUserId()) && $this->userId===$this->_process->getUserId()) {
            if ($this->_process->getProcessing()==0 && $this->_process->getMustTerminate()==0) {
                $this->_canCancelProcess = true;
            }
        } else {
            // si le temps a depasse le temps de deverouillage possible
            if ((time() - $this->_process->getPhaseStartTime())>self::PROCESS_TIME_UNLOCK) {
                if ($this->_process->getProcessing()==0 && $this->_process->getMustTerminate()==0) {
                    $this->_canCancelProcess = true;
                }
            }
        }

        if ($this->_process->getProcessing()>0) {
            $this->_msgList['error'][] = $this->l->t('Une opération "' . self::_getPhaseName($this->_process->getPhase()). '" est déjà en cours...');

            if (!empty($this->_process->getUserId()) && $this->userId!==$this->_process->getUserId()) {
                $this->_msgList['error'][] = $this->l->t('Opération démarrée par l\'utilisateur "' . $this->_process->getUserId() . '".');
            } else {
                $this->_msgList['error'][] = $this->l->t('Veuillez patienter');
            }
        } else if (!empty($this->_process->getUserId()) && $this->userId!==$this->_process->getUserId()) {
            $this->_msgList['error'][] = $this->l->t('Une opération "' . self::_getPhaseName($this->_process->getPhase()) . '" a déjà été démarrée par l\'utilisateur "' . $this->_process->getUserId() . '".');
        } else if ($this->_process->getHasError()>0) {
            if ($this->_process->getMustTerminate()==1) {
                // derniere operation terminee avec des erreurs (afficher le rapport)
                $this->_msgList['error'][] = $this->l->t('L\'opération "' . self::_getPhaseName($this->_process->getPhase()) . '" s\'est terminée avec les erreurs suivantes') . ' : ';
                $this->_msgList['error'][] = $this->_process->getErrorMsg();
            }
        }
    }

    /**
     * Init application folders
     *
     * @throws  \OC\User\NoUserException
     */
    private function _initDematFolders() {
        $appDataDirPath           = $this->_dematpayslipConfigList['app_data_dir']; // dematpayslip directory of user demat (upload, check, report, etc)
        $this->_checkDirUserPath  = $appDataDirPath . DIRECTORY_SEPARATOR . 'check';
        $this->_reportDirUserPath = $appDataDirPath . DIRECTORY_SEPARATOR . 'report';
        $this->_uploadDirUserPath = $appDataDirPath . DIRECTORY_SEPARATOR . 'upload';
        $this->_exportDirUserPath = $appDataDirPath . DIRECTORY_SEPARATOR . 'export';
        $this->_tmpDirUserPath    = $appDataDirPath . DIRECTORY_SEPARATOR . 'tmp';

        // recuperer le dossier de l'utilisateur Nextcloud
        $user = \OC::$server->getUserManager()->get($this->userId);
        if (!$user) {
            $this->_msgList['error'][] = $this->l->t('Le dossier utilisateur "' . $this->userId . '" n\'existe pas.');
        }

        // mounter le dossier de l'utilisateur
        if (count($this->_msgList['error'])<=0) {
            $userHomeAppFilesFullPath = $user->getHome() . DIRECTORY_SEPARATOR . 'files';
            $this->_checkDirFullPath  = $userHomeAppFilesFullPath . DIRECTORY_SEPARATOR . $this->_checkDirUserPath;
            $this->_reportDirFullPath = $userHomeAppFilesFullPath . DIRECTORY_SEPARATOR . $this->_reportDirUserPath;
            $this->_uploadDirFullPath = $userHomeAppFilesFullPath . DIRECTORY_SEPARATOR . $this->_uploadDirUserPath;
            $this->_exportDirFullPath = $userHomeAppFilesFullPath . DIRECTORY_SEPARATOR . $this->_exportDirUserPath;
            $this->_tmpDirFullPath    = $userHomeAppFilesFullPath . DIRECTORY_SEPARATOR . $this->_tmpDirUserPath;

            // monter le dossier de 'utilisateur
            Filesystem::initMountPoints($this->userId);

            // creer le dossier des bulletins
            $isDir = Filesystem::is_dir($this->_dematpayslipConfigList['payslip_dir']);
            if (!$isDir) {
                // creer le dossier des bulletins
                $dirCreated = Filesystem::mkdir($this->_dematpayslipConfigList['payslip_dir']);
                if (!$dirCreated) {
                    $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier des bulletins "' . $this->_dematpayslipConfigList['payslip_dir'] . '".');
                }
            }

            // creer le dossier des bulletins a ne pas indexer
            $isDir = Filesystem::is_dir($this->_payslipNoIndexDir);
            if (!$isDir) {
                // creer le dossier des bulletins a ne pas indexer
                $dirCreated = Filesystem::mkdir($this->_payslipNoIndexDir);
                if (!$dirCreated) {
                    $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier des bulletins non indexes "' . $this->_payslipNoIndexDir . '".');
                }
            }

            // creer le dossier de l'application
            $isDir = Filesystem::is_dir($appDataDirPath);
            if (!$isDir) {
                // creer le dossier de l'application
                $dirCreated = Filesystem::mkdir($appDataDirPath);
                if (!$dirCreated) {
                    $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier de l\'application "' . $appDataDirPath . '".');
                }
            }

            if (count($this->_msgList['error'])<=0) {
                // dossier check
                $isDir = Filesystem::is_dir($this->_checkDirUserPath);
                if (!$isDir) {
                    $dirCreated = \OC\Files\Filesystem::mkdir($this->_checkDirUserPath);
                    if (!$dirCreated) {
                        $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier de l\'application "' . $this->_checkDirUserPath . '".');
                    }
                }

                // dossier report
                $isDir = Filesystem::is_dir($this->_reportDirUserPath);
                if (!$isDir) {
                    $dirCreated = Filesystem::mkdir($this->_reportDirUserPath);
                    if (!$dirCreated) {
                        $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier de l\'application "' . $this->_reportDirUserPath . '".');
                    }
                }

                // dossier upload
                $isDir = Filesystem::is_dir($this->_uploadDirUserPath);
                if (!$isDir) {
                    $dirCreated = Filesystem::mkdir($this->_uploadDirUserPath);
                    if (!$dirCreated) {
                        $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier de l\'application "' . $this->_uploadDirUserPath . '".');
                    }
                }

                // dossier export
                $isDir = Filesystem::is_dir($this->_exportDirUserPath);
                if (!$isDir) {
                    $dirCreated = Filesystem::mkdir($this->_exportDirUserPath);
                    if (!$dirCreated) {
                        $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier de l\'application "' . $this->_exportDirUserPath . '".');
                    }
                }

                // dossier tmp
                $isDir = Filesystem::is_dir($this->_tmpDirUserPath);
                if (!$isDir) {
                    $dirCreated = Filesystem::mkdir($this->_tmpDirUserPath);
                    if (!$dirCreated) {
                        $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier de l\'application "' . $this->_exportDirUserPath . '".');
                    }
                }
            }
        }
    }

    /**
     * Init for demat user
     *
     * @throws \OC\User\NoUserException
     */
    private function _initDematUser() {
        // init
        $appPath = OC_App::getAppPath($this->appName);
        // depends
        require_once $appPath . '/vendor/autoload.php';

        // creer le process s'il n'existe pas
        $dematpayslipProcessList = $this->_dematpayslipProcessMapper->findAll();
        if (empty($dematpayslipProcessList)) {
            $dematpayslipProcess = new DematpayslipProcess();
            $dematpayslipProcess->setProcessing(0);
            $dematpayslipProcess->setPhase(0);
            $dematpayslipProcess->setPhaseStartTime(0);
            $dematpayslipProcess->setPhaseStepNum(0);
            $dematpayslipProcess->setMustTerminate(0);
            $dematpayslipProcess->setHasError(0);
            $dematpayslipProcess->setErrorCode(0);
            $dematpayslipProcess->setErrorMsg('');
            $dematpayslipProcess->setPdfOrigin('');
            $dematpayslipProcess->setCsvOrigin('');
            $dematpayslipProcess->setCsvCheck('');
            $dematpayslipProcess->setUserId('');
            $this->_dematpayslipProcessMapper->insert($dematpayslipProcess);
            $this->_process = $dematpayslipProcess;
        } else {
            $this->_process = $dematpayslipProcessList[0];
        }

        $this->_initDematFolders();
    }

    /**
     * Load config
     */
    private function _loadConfig() {
        $this->_dematpayslipConfigList['payslip_dir']  = \OC::$server->getConfig()->getAppValue($this->appName, 'payslip_dir', 'Bulletins'); // Bulletins
        $this->_dematpayslipConfigList['user_id']      = \OC::$server->getConfig()->getAppValue($this->appName, 'user_id', 'demat');     // demat
        $this->_dematpayslipConfigList['app_data_dir'] = \OC::$server->getConfig()->getAppValue($this->appName, 'app_data_dir', 'dematpayslip'); // dematpayslip directory of user demat (upload, check, report, etc)
        if (empty($this->_dematpayslipConfigList['payslip_dir']) || empty($this->_dematpayslipConfigList['user_id'])) {
            $this->_msgList['error'][] = $this->l->t('Veuillez vérifier la configuration de cette application.');
        }
    }

    /**
     * PageController constructor.
     *
     * @param   string                              $AppName                            App name
     * @param   IRequest                            $request                            Request
     * @param   IConfig                             $config                             Config
     * @param   IL10N                               $l                                  Lang
     * @param   DematpayslipProcessMapper           $dematpayslipProcessMapper          Dematpayslip process mapper
     * @param   DematpayslipProcessSkipMapper       $dematpayslipProcessSkipMapper      Dematpayslip process skip mapper
     * @param   DematpayslipUserMapper              $dematpayslipUserMapper             Dematpayslip user mapper
     * @param   string                              $UserId                             User Id
     */
    public function __construct(
        $AppName,
        IRequest $request,
        IConfig $config,
        IL10N $l,
        DematpayslipProcessMapper $dematpayslipProcessMapper,
        DematpayslipProcessSkipMapper $dematpayslipProcessSkipMapper,
        DematpayslipUserMapper $dematpayslipUserMapper,
        $UserId
    ) {
		parent::__construct($AppName, $request);
		$this->config                          = $config;
		$this->l                               = $l;
		$this->_dematpayslipProcessMapper      = $dematpayslipProcessMapper;
        $this->_dematpayslipProcessSkipMapper  = $dematpayslipProcessSkipMapper;
        $this->_dematpayslipUserMapper         = $dematpayslipUserMapper;
        $this->userId                          = $UserId;

        // load config
        $this->_loadConfig();
	}

	/**
	 * CAUTION: the @Stuff turns off security checks; for this page no admin is
	 *          required and no CSRF check. If you don't know what CSRF is, read
	 *          it up in the docs or you might create a security hole. This is
	 *          basically the only required method to add this exemption, don't
	 *          add it to any other method if you don't exactly know what it does
	 *
	 * @NoAdminRequired
	 * @NoCSRFRequired
     * @UseSession
	 */
	public function index() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematUser();

            $this->_processCheck();
            if (count($this->_msgList['error'])<=0) {
                if ($this->_process->getPhase()==0) {
                    // scan uploaded files
                    $uploadedCSVAndPDFArr = $this->_findUploadedCSVAndPDF();
                    $uploadFileCSVName = $uploadedCSVAndPDFArr['csv'];
                    $uploadFilePDFName = $uploadedCSVAndPDFArr['pdf'];

                    if (!empty($uploadFileCSVName) && !empty($uploadFilePDFName)) {
                        $this->_process->setPhase(1);
                        $this->_process->setCsvOrigin($uploadFileCSVName);
                        $this->_process->setPdfOrigin($uploadFilePDFName);
                        $this->_dematpayslipProcessMapper->update($this->_process);
                    }
                }
            }

            return new TemplateResponse('dematpayslip', 'index', $this->_assignParamsDemat());  // templates/index.php
        } else {
            return $this->shares();
        }
	}

    /**
     * Cancel a process
     * @NoAdminRequired
     */
    public function cancel() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematUser();

            $this->_processCheck();
            if ($this->_canCancelProcess === true) {
                $this->_processCancel();
            }

            return new TemplateResponse('dematpayslip', 'index', $this->_assignParamsDemat());  // templates/index.php
        } else {
            return $this->shares();
        }
    }

    /**
     * Phase Upload
     * @NoAdminRequired
     *
     * @return TemplateResponse
     * @throws \OC\User\NoUserException
     */
    public function upload() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematUser();

            $this->_processCheck();
            if (count($this->_msgList['error'])<=0) {
                $dateTimeNow = new \DateTime('now', \OC::$server->getDateTimeZone()->getTimeZone());

                $this->_process->setProcessing(1);
                $this->_process->setPhase(0);
                $this->_process->setPhaseStartTime($dateTimeNow->getTimestamp());
                $this->_process->setPhaseStepNum(0);
                $this->_process->setHasError(0);
                $this->_process->setErrorCode(0);
                $this->_process->setErrorMsg('');
                $this->_process->setMustTerminate(0);
                $this->_process->setCsvOrigin('');
                $this->_process->setPdfOrigin('');
                $this->_dematpayslipProcessMapper->update($this->_process);

                // check upload CSV
                $uploadFileCSVName    = '';
                $uploadFileCSVTmpName = '';
                if (!isset($_FILES['payslips_csv_file'])) {
                    $errorMsg = $this->l->t('Veuillez sélectionner un fichier CSV.');
                    $this->_processErrorMsg(-1, $errorMsg);
                } else {
                    $uploadFileArr = $_FILES['payslips_csv_file'];
			        //var_dump($uploadFileArr['type']);
                    if (empty($uploadFileArr['name'])) {
                        $errorMsg = $this->l->t('Veuillez sélectionner un fichier CSV.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else if (!in_array($uploadFileArr['type'], array('text/csv', 'application/octet-stream', 'application/vnd.ms-excel'))) {
                        $errorMsg = $this->l->t('Veuillez sélectionner un fichier CSV valide.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else if ($uploadFileArr['size']<=0) {
                        $errorMsg = $this->l->t('Veuillez sélectionner un fichier CSV non vide.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else if (empty($uploadFileArr['tmp_name'])) {
                        $errorMsg = $this->l->t('Impossible d\'uploader ce fichier CSV.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else if ($uploadFileArr['error']!==UPLOAD_ERR_OK) {
                        $errorMsg = $this->l->t('Echec d\'upload du fichier CSV [=' . $uploadFileArr['error'] . '].');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else {
                        // upload success
                        $uploadFileCSVName    = $uploadFileArr['name'];
                        $uploadFileCSVTmpName = $uploadFileArr['tmp_name'];
                    }
                }

                // check upload PDF
                $uploadFilePDFName    = '';
                $uploadFilePDFTmpName = '';
                if (!isset($_FILES['payslips_pdf_file'])) {
                    $errorMsg = $this->l->t('Veuillez sélectionner un fichier PDF.');
                    $this->_processErrorMsg(-1, $errorMsg);
                } else {
                    $uploadFileArr = $_FILES['payslips_pdf_file'];
                    if (empty($uploadFileArr['name'])) {
                        $errorMsg = $this->l->t('Veuillez sélectionner un fichier PDF.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else if ($uploadFileArr['type']!='application/pdf') {
                        $errorMsg = $this->l->t('Veuillez sélectionner un fichier PDF valide.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else if ($uploadFileArr['size']<=0) {
                        $errorMsg = $this->l->t('Veuillez sélectionner un fichier PDF non vide.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else if (empty($uploadFileArr['tmp_name'])) {
                        $errorMsg = $this->l->t('Impossible d\'uploader ce fichier PDF.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else if ($uploadFileArr['error']!==UPLOAD_ERR_OK) {
                        $errorMsg = $this->l->t('Echec d\'upload du fichier PDF [=' . $uploadFileArr['error'] . '].');
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else {
                        // upload success
                        $uploadFilePDFName    = $uploadFileArr['name'];
                        $uploadFilePDFTmpName = $uploadFileArr['tmp_name'];
                    }
                }

                if ($this->_process->getHasError()<=0) {
                    // vider le dossier Upload
                    if (is_dir($this->_uploadDirFullPath)) {
                        self::_removeAllFilesDir($this->_uploadDirFullPath);
                    }

                    // upload CSV
                    $uploadFilePath = $this->_uploadDirFullPath . DIRECTORY_SEPARATOR . $uploadFileCSVName;
                    if (!move_uploaded_file($uploadFileCSVTmpName, $uploadFilePath)) {
                        $errorMsg = $this->l->t('Echec d\'upload du fichier CSV dans le répertoire ' . $uploadFilePath . '.');
                        $this->_processErrorMsg(-1, $errorMsg);
                    }

                    // upload PDF
                    $uploadFilePath = $this->_uploadDirFullPath . DIRECTORY_SEPARATOR . $uploadFilePDFName;
                    if (!move_uploaded_file($uploadFilePDFTmpName, $uploadFilePath)) {
                        $errorMsg = $this->l->t('Echec d\'upload du fichier PDF dans le répertoire ' . $uploadFilePath . '.');
                        $this->_processErrorMsg(-1,$errorMsg);
                    }
                }

                if ($this->_process->getHasError()>0) {
                    if ($this->_process->getPhase()==0) {
                        // scan uploaded files
                        $uploadedCSVAndPDFArr = $this->_findUploadedCSVAndPDF();
                        $uploadFileCSVName = $uploadedCSVAndPDFArr['csv'];
                        $uploadFilePDFName = $uploadedCSVAndPDFArr['pdf'];

                        if (!empty($uploadFileCSVName) && !empty($uploadFilePDFName)) {
                            $this->_process->setPhase(1);
                            $this->_process->setCsvOrigin($uploadFileCSVName);
                            $this->_process->setPdfOrigin($uploadFilePDFName);
                            $this->_process->setUserId($this->userId);
                            $this->_dematpayslipProcessMapper->update($this->_process);
                        }
                    }
                } else {
                    $this->_msgList['success'][] = $this->l->t('Opération "' . self::_getPhaseName($this->_process->getPhase()) . '" terminée avec succès.');
                    $this->_process->setPhase(1);
                    $this->_process->setCsvOrigin($uploadFileCSVName);
                    $this->_process->setPdfOrigin($uploadFilePDFName);
                    $this->_process->setUserId($this->userId);
                }

                $this->_process->setProcessing(0);
                $this->_dematpayslipProcessMapper->update($this->_process);
            }

            return new TemplateResponse('dematpayslip', 'index', $this->_assignParamsDemat());  // templates/index.php
        } else {
            return $this->shares();
        }
    }

    /**
     * Phase Check
     * @NoAdminRequired
     *
     * @return  TemplateResponse
     * @throws  \OC\User\NoUserException
     * @throws  \setasign\Fpdi\PdfParser\CrossReference\CrossReferenceException
     * @throws  \setasign\Fpdi\PdfParser\Filter\FilterException
     * @throws  \setasign\Fpdi\PdfParser\PdfParserException
     * @throws  \setasign\Fpdi\PdfParser\Type\PdfTypeException
     * @throws  \setasign\Fpdi\PdfReader\PdfReaderException
     */
    public function check() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematUser();
            $this->_processCheck();
            if ($this->_process->getPhase() < self::PHASE_ID_CHECK) {
                $this->_msgList['error'][] = $this->l->t('Opération impossible : phase en cours "' . self::_getPhaseName($this->_process->getPhase()) . '".');
            }

            if (count($this->_msgList['error']) <= 0) {
                $this->_processFailOnFirstError = false;
                $dateTimeNow = new \DateTime('now', \OC::$server->getDateTimeZone()->getTimeZone());

                $this->_process->setProcessing(1);
                $this->_process->setPhase(1);
                $this->_process->setPhaseStartTime($dateTimeNow->getTimestamp());
                $this->_process->setPhaseStepNum(0);
                $this->_process->setMustTerminate(0);
                $this->_process->setHasError(0);
                $this->_process->setErrorCode(0);
                $this->_process->setErrorMsg('');
                $this->_process->setUserId($this->userId);
                $this->_dematpayslipProcessMapper->update($this->_process);

                // vider les lignes du process a sauter
                $this->_dematpayslipProcessSkipMapper->deleteAllByIdProcess($this->_process->getId());

                // vider le dossier Check
                if (is_dir($this->_checkDirFullPath)) {
                    self::_removeAllFilesDir($this->_checkDirFullPath);
                }

                $uploadFilePDFName = $this->_process->getPdfOrigin();
                $uploadFileCSVName = $this->_process->getCsvOrigin();

                // log file
                $logFilePath = $this->_checkDirUserPath . DIRECTORY_SEPARATOR . $this->_process->getPhase() . '_log';
                $this->_logFile = Filesystem::fopen($logFilePath, 'w+');
                $this->_logCsvCols(array('date', 'heure', 'niveau', 'message', 'siret', 'matricule', 'nom_prenom', 'zip_city', 'date_debut', 'date_fin', 'email'));
                $this->_logCsvLine('## ' .$this->l->t('Début du processus de vérification.') . ' ##', self::LOG_DEBUG);

                $csvCheckFileName = $dateTimeNow->format('Y-m-d_H-i-s') . '_' . $uploadFileCSVName;

                $this->_logCsvLine('## ' .$this->l->t('Début de vérification des champs du fichier CSV.') . ' ##', self::LOG_DEBUG);
                // pdf read and parse
                $pdfFilePath = $this->_uploadDirFullPath . DIRECTORY_SEPARATOR . $uploadFilePDFName;
                $pdfPages = array();
                if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                    $this->_logCsvLine($this->l->t('Lecture du fichier PDF.'), self::LOG_DEBUG);
                    $pdfParser = new \Smalot\PdfParser\Parser();
                    $pdf = $pdfParser->parseFile($pdfFilePath);
                    $pdfPages = $pdf->getPages();
                    $pdfPageNb = count($pdfPages);
                    if ($pdfPageNb <= 0) {
                        $errorMsg = $this->l->t('Aucune page dans le PDF "' . $uploadFilePDFName . '".');
                        $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                        $this->_processErrorMsg(-1, $errorMsg);
                    }
                }

                // csv read and load all lines
                $csvTodoLineList = array();
                if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                    $this->_logCsvLine($this->l->t('Ouverture du fichier CSV.'), self::LOG_DEBUG);
                    $csvFilePath = $this->_uploadDirFullPath . DIRECTORY_SEPARATOR . $uploadFileCSVName;
                    $csvFile = fopen($csvFilePath, 'r');
                    if (!$csvFile) {
                        $errorMsg = $this->l->t('Impossible d\'ouvrir le fichier CSV "' . $uploadFileCSVName . '".');
                        $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                        $this->_processErrorMsg(-1, $errorMsg);
                    } else {
                        $csvLineNum = 0;
                        $csvLineNumBegin = 0;
                        $this->_csvEncodeFrom = 'ISO-8859-1';
                        $csvConvert = false;
                        if (!empty($this->_csvEncodeFrom) && $this->_csvEncodeFrom!=$this->_csvEncodeTo) {
                            $csvConvert = true;
                        }
                        $this->_logCsvLine($this->l->t('Lecture des lignes du fichier CSV.'), self::LOG_DEBUG);
                        while ($csvLine = fgetcsv($csvFile, $this->_csvLength, $this->_csvDelimiter, $this->_csvEnclosure)) {
                            $this->_logCsvLine('-- ' . $this->l->t('Début de ligne CSV : ' .  ($csvLineNum + 1)) . ' --', self::LOG_DEBUG);

                            if ($csvLineNum >= $csvLineNumBegin) {
                                if ($csvConvert === true) {
                                    foreach ($csvLine as $csvColNum => $csvValue) {
                                        $csvLine[$csvColNum] = mb_convert_encoding(trim($csvValue), $this->_csvEncodeTo,  $this->_csvEncodeFrom);
                                    }
                                }
                                $matriculeOri = $csvLine[1];
                                $startDate    = date('Y-m-d', strtotime($csvLine[13]));
                                $endDate      = date('Y-m-d', strtotime($csvLine[14]));
                                $editionDate  = date('Y-m-d', strtotime($csvLine[17]));

                                // agent user
                                $agentUser = new DematpayslipUser();
                                $agentUser->setId(0);
                                $agentUser->setUserId('');
                                $agentUser->setSiret($csvLine[0]);
                                $agentUser->setMatricule(str_pad($matriculeOri, '8', '0', STR_PAD_LEFT));
                                $agentUser->setLastname($csvLine[3]);
                                $agentUser->setFirstname($csvLine[4]);
                                $agentUser->setEmail($csvLine[5]);
                                $agentUser->setZipCity($csvLine[10]);

                                // verifier l'existance de l'email et autres champs (regex)
                                $this->_logCsvLine($this->l->t('Ligne CSV'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                if (empty($agentUser->getEmail())) {
                                    $warningMsg = $this->l->t('Le fichier CSV "' . $uploadFileCSVName . '" contient un champ email vide [ligne=' . ($csvLineNum + 1) . '].');
                                    $this->_logCsvLine($warningMsg, self::LOG_WARNING, $agentUser, $startDate, $endDate);
                                    $this->_processWarningMsg($warningMsg);

                                    // sauter cette ligne dans la suite du process de verification et d'indexation
                                    //$dematpayslipProcessSkipLineArr = $this->_dematpayslipProcessSkipMapper->findAllByIdProcessAndIdLine($this->_process->getId(), $csvLineNum);
                                    //if (empty($dematpayslipProcessSkipLineArr)) {
                                    //    $dematpayslipProcessSkip = new DematpayslipProcessSkip();
                                    //    $dematpayslipProcessSkip->setIdProcess($this->_process->getId());
                                    //    $dematpayslipProcessSkip->setIdLine($csvLineNum);
                                    //    $this->_dematpayslipProcessSkipMapper->insert($dematpayslipProcessSkip);
                                    //}
                                } else {
                                    $userManager = \OC::$server->getUserManager();
                                    $userFindByEmailList = $userManager->getByEmail($agentUser->getEmail());

                                    // /!\ remove all users not available (FIX for multiple users with SAML)
                                    if (self::USER_REMOVE_UNKNOWN == 1) {
                                        $userFindByEmailList = self::_removeUnknownUser($userFindByEmailList);
                                    }

                                    if (count($userFindByEmailList) != 1) {
                                        $warningMsg = $this->l->t('Le fichier CSV "' . $uploadFileCSVName . '" contient un email "' . $agentUser->getEmail() . '" non valide ou non lié à un unique utilisateur Nextcloud [ligne=' . ($csvLineNum + 1) . '].');
                                        $this->_logCsvLine($warningMsg, self::LOG_WARNING, $agentUser, $startDate, $endDate);
                                        $this->_processWarningMsg($warningMsg);

                                        $dematpayslipProcessSkipLineArr = $this->_dematpayslipProcessSkipMapper->findAllByIdProcessAndIdLine($this->_process->getId(), $csvLineNum);
                                        if (empty($dematpayslipProcessSkipLineArr)) {
                                            $dematpayslipProcessSkip = new DematpayslipProcessSkip();
                                            $dematpayslipProcessSkip->setIdProcess($this->_process->getId());
                                            $dematpayslipProcessSkip->setIdLine($csvLineNum);
                                            $this->_dematpayslipProcessSkipMapper->insert($dematpayslipProcessSkip);
                                        }
                                    }
                                }

                                if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                                    $csvTodoLineList[] = array(
                                        'siret'         => $agentUser->getSiret(),
                                        'matricule_ori' => $matriculeOri,
                                        'lastname'      => $agentUser->getLastname(),
                                        'firstname'     => $agentUser->getFirstname(),
                                        'zip_city'      => $agentUser->getZipCity(),
                                        'start_date'    => $startDate,
                                        'end_date'      => $endDate,
                                        'edition_date'  => $editionDate,
                                        'matricule'     => $agentUser->getMatricule(),
                                        'fullname'      => $agentUser->getLastname() . ' ' . $agentUser->getFirstname(),
                                        'email'         => $agentUser->getEmail(),
                                        'line_num'      => $csvLineNum + 1,
                                        'pdf_pages'     => '',
                                    );
                                }
                            }

                            if ($this->_processFailOnFirstError===true && $this->_process->getHasError()>0) {
                                $this->_logCsvLine('-- ' . $this->l->t('Fin de ligne CSV : ' .  ($csvLineNum + 1)) . ' --', self::LOG_ERROR);
                                break;
                            } else {
                                $this->_logCsvLine('-- ' . $this->l->t('Fin de ligne CSV : ' .  ($csvLineNum + 1)) . ' --', self::LOG_DEBUG);
                            }

                            $csvLineNum++;
                        }
                        fclose($csvFile);
                    }
                }
                if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                    if (count($csvTodoLineList) <= 0) {
                        $errorMsg = $this->l->t('Aucune ligne dans le fichier CSV "' . $uploadFileCSVName . '".');
                        $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                        $this->_processErrorMsg(-1, $errorMsg);
                    }
                }
                $this->_logCsvLine('## ' .$this->l->t('Fin de vérification des champs du fichier CSV.') . ' ##', self::LOG_DEBUG);

                if ($this->_process->getHasError()>0)   $this->_processFailOnFirstError=true;
                if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                    $this->_logCsvLine('## ' .$this->l->t('Début de l\'association des pages du PDF aux différents agents.') . ' ##', self::LOG_DEBUG);

                    $this->_logCsvLine($this->l->t('Création du fichier CSV des assoications des pages de bulletins de paie aux différents agents.'), self::LOG_DEBUG);
                    $csvCheckFilePath = $this->_checkDirFullPath . DIRECTORY_SEPARATOR . $csvCheckFileName;
                    $csvCheckFile = fopen($csvCheckFilePath, 'w+');
                    if (!$csvCheckFile) {
                        $errorMsg = $this->l->t('Impossible de créer le fichier CSV "' . $csvCheckFileName . '".');
                        $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                        $this->_processErrorMsg(-1, $errorMsg);
                    }

                    if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                        $checkLastValues = array(
                            'siret'     => '',
                            'matricule' => '',
                            'fullname'  => '',
                            'zip_city'  => '',
                        );

                        $checkLastLineNum = -1;

                        // pour chaque page du PDF
                        $this->_logCsvLine($this->l->t('Lecture des pages du fichier PDF.'), self::LOG_DEBUG);
                        $pdfPageList = array();
                        foreach ($pdfPages as $pdfPageNum => $pdfPage) {
                            $this->_logCsvLine('-- ' . $this->l->t('Page PDF en cours : ' .  ($pdfPageNum + 1) . ' --'), self::LOG_DEBUG);

                            // recuperer les champs textes dans le PDF et enlever les espaces
                            $this->_logCsvLine($this->l->t('Récupération des champs textes du fichier PDF.'), self::LOG_DEBUG);
                            $pdfPageTextList = array();
                            $pdfPageTextArr = $pdfPage->getTextArray();
                            foreach ($pdfPageTextArr as $pdfPageTextKey => $pdfPageText) {
                                // formater les champs textes du PDF (enlever les espaces)
                                $pdfPageTextList[] = trim($pdfPageText);
                            }

                            // pour chaque ligne du CSV restant a traiter
                            $this->_logCsvLine($this->l->t('Lecture des lignes fichier CSV.'), self::LOG_DEBUG);
                            $csvTodoLineNum = 0;
                            $csvTodoLine = array();
                            $checkCurrentValues = array();
                            $pdfPageValues = array();
                            $agentFound = false;
                            foreach ($csvTodoLineList as $csvTodoLineNum => $csvTodoLine) {
                                $this->_logCsvLine('-- ' . $this->l->t('Début de ligne CSV : ' .  ($csvTodoLineNum + 1)) . ' --', self::LOG_DEBUG);

                                // recuperer les valeurs de controle
                                $checkCurrentValues = array(
                                    'siret'     => $csvTodoLine['siret'],
                                    'matricule' => $csvTodoLine['matricule'],
                                    'fullname'  => $csvTodoLine['fullname'],
                                    'zip_city'  => $csvTodoLine['zip_city'],
                                );

                                $pdfPageValues = array();

                                // pour chaque champ CSV a controler
                                foreach ($checkCurrentValues as $checkField => $checkCurrentValue) {
                                    // verifier d'abord aux emplacements previsibles du PDF
                                    if ($checkField == 'siret') {
                                        // siret
                                        if (isset($this->_payslipPDFSearchFieldList[$checkField]) && !empty($this->_payslipPDFSearchFieldList[$checkField])) {
                                            foreach ($this->_payslipPDFSearchFieldList[$checkField] as $payslipPDFSearchFieldIndex) {
                                                $matchList = array();
                                                if (preg_match('#SIRET:\s(\d+)\s\d+$#', $pdfPageTextList[$payslipPDFSearchFieldIndex], $matchList)) {
                                                   if ($matchList[1] == $checkCurrentValue) {
                                                        $pdfPageValues[$checkField] = $checkCurrentValue;
                                                        continue;
                                                    }
                                                }
                                            }
                                        }
                                    } else {
                                        // siret, matricule, fullname, zip_city
                                        if (isset($this->_payslipPDFSearchFieldList[$checkField]) && !empty($this->_payslipPDFSearchFieldList[$checkField])) {
                                            foreach ($this->_payslipPDFSearchFieldList[$checkField] as $payslipPDFSearchFieldIndex) {
                                                if ($pdfPageTextList[$payslipPDFSearchFieldIndex] == $checkCurrentValue) {
                                                    $pdfPageValues[$checkField] = $checkCurrentValue;
                                                    break;
                                                }
                                            }
                                        }
                                    }

                                    if (!isset($pdfPageValues[$checkField])) {
                                        // si on a pas trouve aux emplacement previsibles du PDF
                                        // pour chaque champ texte de la page du PDF
                                        foreach ($pdfPageTextList as $pdfPageTextKey => $pdfPageText) {
                                            // formater les champs textes du PDF (enlever les espaces)
                                            $pdfPageTextValue = trim($pdfPageText);

                                            if ($checkField == 'siret') {
                                                $matchList = array();
                                                if (preg_match('#SIRET:\s(\d+)\s\d+$#', $pdfPageTextValue, $matchList)) {
                                                    if ($matchList[1] == $checkCurrentValue) {
                                                        $pdfPageValues[$checkField] = $checkCurrentValue;
                                                        break;
                                                    }
                                                }
                                            } else {
                                                // siret, matricule, fullname, zip_city
                                                if ($pdfPageTextValue == $checkCurrentValue) {
                                                    $pdfPageValues[$checkField] = $checkCurrentValue;
                                                    break;
                                                }
                                            }
                                        }
                                    }
                                }

                                // verifie si on a trouve un agent correspondant a la ligne du fichier CSV en cours
                                $agentFound = false;
                                if (!empty($pdfPageValues)) {
                                    foreach ($checkCurrentValues as $checkField => $checkCurrentValue) {
                                        if (!isset($pdfPageValues[$checkField]) || empty($pdfPageValues[$checkField])) {
                                            $agentFound = false;
                                            break;
                                        } else {
                                            $agentFound = true;
                                        }
                                    }
                                }
                                if ($agentFound === true) {
                                    break;
                                }

                                $this->_logCsvLine('-- ' . $this->l->t('Fin de ligne CSV : ' .  ($csvTodoLineNum + 1)) . ' --', self::LOG_DEBUG);
                            }

                            if ($agentFound !== true) {
                                $errorMsg = $this->l->t('Aucun agent trouve dans la page ' . ($pdfPageNum + 1) . ' du PDF correspondant aux lignes du fichier CSV.');
                                $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                $this->_processErrorMsg(-1, $errorMsg);
                                break;
                            } else {
                                // si on a bien trouve l'agent correspondant a la page du PDF
                                $siret = $csvTodoLine['siret'];
                                $matricule = $csvTodoLine['matricule'];
                                $period = $csvTodoLine['start_date'] . '_' . $csvTodoLine['end_date'];
                                $agentKey = $siret . '-' . $matricule;
                                $this->_logCsvLine($this->l->t('Agent trouvé [siret=' . $siret . ', matricule=' . $matricule . ', period=' . $period . '].'), self::LOG_DEBUG);

                                $newAgent = false;
                                foreach ($checkLastValues as $checkLastField => $checLastValue) {
                                    if ($pdfPageValues[$checkLastField] != $checLastValue) {
                                        $newAgent = true;
                                        break;
                                    }
                                }

                                if ($newAgent === true) {
                                    $this->_logCsvLine($this->l->t('C\'est un nouvel agent.'), self::LOG_DEBUG);
                                    $this->_logCsvLine($this->l->t('Ajout de la page ' . ($pdfPageNum + 1) .' du PDF des bulletins de paie de l\'agent.'), self::LOG_DEBUG);

                                    $pdfPageList[$agentKey] = array('page_num_list' => array($pdfPageNum + 1), 'period' => $period);

                                    if ($checkLastLineNum >= 0) {
                                        // mettre ligne CSV dans fichier CSV qui va servir pour l'indexation (y ajouter les numeros de pages du fichier PDF)
                                        $checkAgentKey = $csvTodoLineList[$checkLastLineNum]['siret'] . '-' . $csvTodoLineList[$checkLastLineNum]['matricule'];
                                        $csvTodoLineList[$checkLastLineNum]['pdf_pages'] = implode('-', $pdfPageList[$checkAgentKey]['page_num_list']);
                                        fputcsv($csvCheckFile, $csvTodoLineList[$checkLastLineNum], $this->_csvDelimiter, $this->_csvEnclosure);
                                        unset($csvTodoLineList[$checkLastLineNum]); // enlever ligne de l'ancien agent
                                    }

                                    // sauvegarder le numero de la ligne CSV lie a l'ancien agent
                                    $checkLastLineNum = $csvTodoLineNum;

                                    // mettre les nouvelles valeurs dans les anciennes valeurs a verifier
                                    $checkLastValues = $checkCurrentValues;
                                } else {
                                    $this->_logCsvLine($this->l->t('Ajout de la page ' . ($pdfPageNum + 1) .' du PDF des bulletins de paie de l\'agent.'), self::LOG_DEBUG);
                                    $pdfPageList[$agentKey]['page_num_list'][] = $pdfPageNum + 1;
                                }

                                // cas de la derniere page du PDF
                                if ($pdfPageNum == $pdfPageNb - 1) {
                                    if ($checkLastLineNum >= 0) {
                                        // mettre ligne CSV dans fichier CSV qui va servir pour l'indexation (y ajouter les numeros de pages du fichier PDF)
                                        $checkAgentKey = $csvTodoLineList[$checkLastLineNum]['siret'] . '-' . $csvTodoLineList[$checkLastLineNum]['matricule'];
                                        $csvTodoLineList[$checkLastLineNum]['pdf_pages'] = implode('-', $pdfPageList[$checkAgentKey]['page_num_list']);
                                        fputcsv($csvCheckFile, $csvTodoLineList[$checkLastLineNum], $this->_csvDelimiter, $this->_csvEnclosure);
                                        unset($csvTodoLineList[$checkLastLineNum]); // enlever ligne de l'ancien agent
                                    }
                                }
                            }

                            $this->_logCsvLine('-- ' . $this->l->t('Fin de page PDF : ' .  ($pdfPageNum + 1) . ' --'), self::LOG_DEBUG);
                        }
                        fclose($csvCheckFile);

                        if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                            // Si on n'a pas traite toutes les lignes du CSV
                            if (count($csvTodoLineList) > 0) {
                                $errorMsgLines = array();
                                $errorMsgLines[] = $this->l->t('Des lignes du fichier CSV "' . $uploadFileCSVName . '" n\'ont pas été traitées') . ' : ';
                                foreach ($csvTodoLineList as $csvTodoLineNum => $csvTodoLine) {
                                    $errorMsgLines[] = ' - ' . $this->l->t('la ligne ' . $csvTodoLine['line_num']);
                                }
                                $this->_logCsvLine(implode("\n", $errorMsgLines), self::LOG_ERROR);
                                $this->_processErrorMsg(-1, implode('<br />', $errorMsgLines));
                            }
                        }
                        $this->_logCsvLine('## ' .$this->l->t('Fin de l\'association des pages du PDF aux différents agents.') . ' ##', self::LOG_DEBUG);
                    }

                    // Phase - decoupage des pages du fichier PDF
                    if ($this->_process->getHasError()>0)   $this->_processFailOnFirstError = true;
                    if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                        $this->_logCsvLine('## ' . $this->l->t('Début de découpage des pages du fichier PDF.') . ' ##', self::LOG_DEBUG);

                        // ouvrir le fichier CSV du dossier "check"
                        $this->_logCsvLine($this->l->t('Ouverture des lignes du fichier CSV contenant les pages des bulletins de paie à découper.'), self::LOG_DEBUG);
                        $csvCheckFilePath = $this->_checkDirFullPath . DIRECTORY_SEPARATOR . $csvCheckFileName;
                        $csvCheckFile = fopen($csvCheckFilePath, 'r');
                        if (!$csvCheckFile) {
                            $errorMsg = $this->l->t('Impossible d\'ouvrir le fichier CSV "' . $csvCheckFileName . '".');
                            $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                            $this->_processErrorMsg(-1, $errorMsg);
                        }

                        if ($this->_processFailOnFirstError===false || $this->_process->getHasError()<=0) {
                            // pour chaque ligne CSV
                            $this->_logCsvLine($this->l->t('Lecture des lignes fichier CSV.'), self::LOG_DEBUG);
                            $csvCheckLineNum = 0;
                            while ($csvCheckLine = fgetcsv($csvCheckFile, $this->_csvLength, $this->_csvDelimiter, $this->_csvEnclosure)) {
                                $this->_logCsvLine('-- ' . $this->l->t('Début de ligne CSV : ' .  ($csvCheckLineNum + 1)) . ' --', self::LOG_DEBUG);
                                //$csvCheckArr[] = array(
                                //    'siret'         => $csvCheckLine[0],
                                //    'matricule_ori' => $csvCheckLine[1],
                                //    'lastname'      => $csvCheckLine[2],
                                //    'firstname'     => $csvCheckLine[3],
                                //    'zip_city'      => $csvCheckLine[4],
                                //    'start_date'    => $csvCheckLine[5],
                                //    'end_date'      => $csvCheckLine[6],
                                //    'edition_date'  => $csvCheckLine[7],
                                //    'matricule'     => $csvCheckLine[8],
                                //    'fullname'      => $csvCheckLine[9],
                                //    'email'         => $csvCheckLine[10],
                                //    'line_num'      => $csvCheckLine[11],
                                //    'pdf_pages'     => $csvCheckLine[12],
                                //);

                                // get CSV data values
                                $csvCheckMatricule = $csvCheckLine[8];
                                $pdfPageNumList = explode('-', $csvCheckLine[12]);
                                $this->_logCsvLine($this->l->t('Découpage des pages du PDF pour le bulletin de paie de l\'agent [matricule=' . $csvCheckMatricule . '].'), self::LOG_DEBUG);

                                // ajout des pages
                                $this->_logCsvLine($this->l->t('Ajout des pages PDF au bulletin de paie.'), self::LOG_DEBUG);
                                $pdfPayslip = new \setasign\Fpdi\Fpdi();
                                foreach ($pdfPageNumList as $pdfPageNum) {
                                    $this->_logCsvLine($this->l->t('Ajout de la page ' . $pdfPageNum . '.'), self::LOG_DEBUG);
                                    $pdfPayslip->AddPage();
                                    $pdfPayslip->setSourceFile($pdfFilePath);
                                    $tplPayslipId = $pdfPayslip->importPage($pdfPageNum);
                                    $pdfPayslip->useTemplate($tplPayslipId);
                                }

                                /// mettre la fiche de paie dans le dossier "check"
                                $this->_logCsvLine($this->l->t('Créer le PDF du bulletin de paie.'), self::LOG_DEBUG);
                                $pdfPayslip->Output('F', $this->_checkDirFullPath . DIRECTORY_SEPARATOR . ($csvCheckLineNum + 1) . '_' . $csvCheckMatricule . '.pdf');

                                $this->_logCsvLine('-- ' . $this->l->t('Fin de ligne CSV : ' .  ($csvCheckLineNum + 1)) . ' --', self::LOG_DEBUG);

                                $csvCheckLineNum++;
                            }
                            fclose($csvCheckFile);
                        }

                        $this->_logCsvLine('## ' . $this->l->t('Fin de découpage des pages du fichier PDF.') . ' ##', self::LOG_DEBUG);
                    }
                }

                if ($this->_process->getHasError() <= 0) {
                    $successMsg = $this->l->t('Opération "' . self::_getPhaseName($this->_process->getPhase()) . '" terminée avec succès.');
                    $this->_logCsvLine($successMsg, self::LOG_SUCCESS);
                    $this->_msgList['success'][] = $successMsg;
                    $this->_process->setPhase(2);
                    $this->_process->setCsvCheck($csvCheckFileName);
                }

                $this->_logCsvLine('## ' .$this->l->t('Fin du processus de vérification.') . ' ##', self::LOG_DEBUG);
                fclose($this->_logFile);
                $renamed = Filesystem::rename($logFilePath, $logFilePath . '.csv');
                $this->_process->setProcessing(0);
                $this->_dematpayslipProcessMapper->update($this->_process);
            }

            return new TemplateResponse('dematpayslip', 'index', $this->_assignParamsDemat());
        } else {
            return $this->shares();
        }
    }

    /**
     * Phase Indexation
     * @NoAdminRequired
     *
     * @param   int                 $retry
     * @return  TemplateResponse
     *
     * @throws NotFoundException
     * @throws \OCP\Files\InvalidPathException
     * @throws \OC\User\NoUserException
     * @throws \setasign\Fpdi\PdfParser\CrossReference\CrossReferenceException
     * @throws \setasign\Fpdi\PdfParser\Filter\FilterException
     * @throws \setasign\Fpdi\PdfParser\PdfParserException
     * @throws \setasign\Fpdi\PdfParser\Type\PdfTypeException
     * @throws \setasign\Fpdi\PdfReader\PdfReaderException
     */
    public function indexation(int $retry) {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematUser();

            if ($retry > 0) {
                $this->_process->setHasError(0);
                $this->_process->setErrorCode(0);
                $this->_process->setErrorMsg('');
            }

            $this->_processCheck();
            if ($this->_process->getPhase() < self::PHASE_ID_INDEXATION) {
                $this->_msgList['error'][] = $this->l->t('Opération impossible : phase en cours "' . self::_getPhaseName($this->_process->getPhase()) . '".');
            }

            if (count($this->_msgList['error']) <= 0) {
                $phaseStartDateTime = new \DateTime('now', \OC::$server->getDateTimeZone()->getTimeZone());
                $phaseStartTime = $phaseStartDateTime->getTimestamp();

                // log file
                $logFilePath = $this->_reportDirUserPath . DIRECTORY_SEPARATOR . $phaseStartDateTime->format('Y-m-d_H-i-s') . '_' . $this->_process->getPhase() . '_log';
                $this->_logFile = Filesystem::fopen($logFilePath, 'w+');
                $this->_logCsvCols(array('date', 'heure', 'niveau', 'message', 'siret', 'matricule', 'nom_prenom', 'zip_city', 'date_debut', 'date_fin', 'email'));
                $this->_logCsvLine('## ' . $this->l->t('Début du processus d\'indexation.') . ' ##', self::LOG_DEBUG);

                $this->_process->setProcessing(1);
                $this->_process->setPhase(2);
                $this->_process->setPhaseStartTime($phaseStartTime);
                if (!($retry > 0)) {
                    $this->_process->setPhaseStepNum(0);
                }
                $this->_process->setMustTerminate(1);
                $this->_process->setHasError(0);
                $this->_process->setErrorCode(0);
                $this->_process->setErrorMsg('');
                $this->_process->setUserId($this->userId);
                $this->_dematpayslipProcessMapper->update($this->_process);

                $payslipsDirPath = $this->_dematpayslipConfigList['payslip_dir'];
                $csvCheckFileName = $this->_process->getCsvCheck();

                // recuperer et monter le dossier de l'utilisateur Nextcloud
                $this->_logCsvLine($this->l->t('Récupération et montage du dossier de l\'utilisateur Nextcloud.'), self::LOG_DEBUG);
                $userHomeFilesDirPath = '';
                if ($this->_process->getHasError() <= 0) {
                    $user = \OC::$server->getUserManager()->get($this->userId);
                    if (!$user) {
                        $errorMsg = $this->l->t('Le dossier utilisateur "' . $this->userId . '" n\'existe pas.');
                        $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                        $this->_processErrorMsg(-1, $errorMsg);
                    }

                    $userHomeFilesDirPath = $user->getHome() . DIRECTORY_SEPARATOR . 'files';
                }
                if (empty($userHomeFilesDirPath) || !is_dir($userHomeFilesDirPath)) {
                    $errorMsg = $this->l->t('Le dossier utilisateur "' . $this->userId . '" est introuvable.');
                    $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                    $this->_processErrorMsg(-1, $errorMsg);
                }

                // verifier l'existance du dossier des bulletins
                $this->_logCsvLine($this->l->t('Vérification de l\'existance du dossier des bulletins.'), self::LOG_DEBUG);
                if ($this->_process->getHasError() <= 0) {
                    $isDir = Filesystem::is_dir($payslipsDirPath);
                    if (!$isDir) {
                        $errorMsg = $this->l->t('Le dossier des bulletins "' . $payslipsDirPath . '" n\'existe pas.');
                        $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                        $this->_processErrorMsg(-1, $errorMsg);
                    }
                }

                // verifier l'existance du dossier des bulletins non indexes
                $this->_logCsvLine($this->l->t('Vérification de l\'existance du dossier des bulletins non indexes.'), self::LOG_DEBUG);
                if ($this->_process->getHasError() <= 0) {
                    $isDir = Filesystem::is_dir($this->_payslipNoIndexDir);
                    if (!$isDir) {
                        $errorMsg = $this->l->t('Le dossier des bulletins non indexes "' . $this->_payslipNoIndexDir . '" n\'existe pas.');
                        $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                        $this->_processErrorMsg(-1, $errorMsg);
                    }
                }

                if ($this->_process->getHasError() <= 0) {
                    $phaseStep = 'csv_check';
                    $csvCheckLineNum = 0;
                    $retryCsvNumLine = 0;
                    if ($retry > 0) {
                        if ($this->_process->getPhaseStep() == $phaseStep) {
                            $this->_process->setPhaseStep('');
                        }
                        $retryCsvNumLine = $this->_process->getPhaseStepNum();
                        $this->_logCsvLine($this->l->t('Reprise à la ligne ' . ($retryCsvNumLine+1) . ' du fichier CSV créé lors de la phase de vérification.'), self::LOG_DEBUG);
                    }
                    if (empty($this->_process->getPhaseStep())) {
                        $rootFolder   = \OC::$server->getRootFolder();
                        $shareManager = \OC::$server->getShareManager();
                        $userManager  = \OC::$server->getUserManager();
                        $userFolder   = $rootFolder->getUserFolder($this->userId);

                        // recuperer tous les partages realises par cet utilisateur
                        $payslipShareNode = $userFolder->get($payslipsDirPath);
                        $payslipShareList = $shareManager->getSharesInFolder($this->userId, $payslipShareNode);
                        $payslipMatriculeShareNodeIdList = array();
                        if (count($payslipShareList) > 0) {
                            foreach ($payslipShareList as $payslipShareNodeId => $payslipShareArr) {
                                $payslipMatriculeShareNodeIdList[] = $payslipShareNodeId;
                            }
                        }

                        // ouverture du fichier CSV cree lors de l'etape de verification
                        $this->_logCsvLine($this->l->t('Ouverture du fichier CSV créé lors de l\'étape de vérification.'), self::LOG_DEBUG);
                        $csvCheckFilePath = $this->_checkDirFullPath . DIRECTORY_SEPARATOR . $csvCheckFileName;
                        $csvCheckFile = fopen($csvCheckFilePath, 'r');
                        if (!$csvCheckFile) {
                            $errorMsg = $this->l->t('Impossible d\'ouvrir le fichier CSV "' . $csvCheckFileName . '".');
                            $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                            $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                        }

                        if ($this->_process->getHasError() <= 0) {
                            $this->_logCsvLine($this->l->t('Lecture du fichier PDF.'), self::LOG_DEBUG);
                            $pdfOriginFileFullPath = $this->_uploadDirFullPath . DIRECTORY_SEPARATOR . $this->_process->getPdfOrigin();

                            // pdf payslip file path
                            $pdfPayslipFilePath = '';

                            // creer le PDF des bulletins de paie a ne pas indexer (ne pas deposer dans le dossier partage de l'agent)
                            $pdfPayslipSkip = null;
                            $payslipSkipPDFPath = $this->_payslipNoIndexDir . DIRECTORY_SEPARATOR .  $phaseStartDateTime->format('Y-m-d_H-i-s') . '_skip';

                            // identifier les lignes du fichier CSV a ne pas indexer
                            $dematpayslipProcessSkipLineIdList = array();
                            $dematpayslipProcessSkipList = $this->_dematpayslipProcessSkipMapper->findAllByIdProcess($this->_process->getId());
                            foreach ($dematpayslipProcessSkipList as $dematpayslipProcessSkip) {
                                $dematpayslipProcessSkipLineIdList[] = $dematpayslipProcessSkip->getIdLine();
                            }

                            // payslip archive mapper
                            $dematpayslipArchiveMapper = new DematpayslipArchiveMapper(\OC::$server->getDatabaseConnection());

                            $this->_logCsvLine($this->l->t('Lecture des lignes du fichier CSV créé lors de l\'étape de vérification.'), self::LOG_DEBUG);
                            while ($csvCheckLine = fgetcsv($csvCheckFile, $this->_csvLength, $this->_csvDelimiter, $this->_csvEnclosure)) {
                                $this->_logCsvLine('-- ' . $this->l->t('Début de ligne CSV : ' . ($csvCheckLineNum + 1)) . ' --', self::LOG_DEBUG);

                                if ($csvCheckLineNum >= $retryCsvNumLine) {
                                    //$csvCheckArr[] = array(
                                    //    'siret'         => $csvCheckLine[0],
                                    //    'matricule_ori' => $csvCheckLine[1],
                                    //    'lastname'      => $csvCheckLine[2],
                                    //    'firstname'     => $csvCheckLine[3],
                                    //    'zip_city'      => $csvCheckLine[4],
                                    //    'start_date'    => $csvCheckLine[5],
                                    //    'end_date'      => $csvCheckLine[6],
                                    //    'edition_date'  => $csvCheckLine[7],
                                    //    'matricule'     => $csvCheckLine[8],
                                    //    'fullname'      => $csvCheckLine[9],
                                    //    'email'         => $csvCheckLine[10],
                                    //    'line_num'      => $csvCheckLine[11],
                                    //    'pdf_pages'     => $csvCheckLine[12],
                                    //);

                                    // get CSV data values
                                    $startDate      = $csvCheckLine[5];
                                    $endDate        = $csvCheckLine[6];
                                    $editionDate    = $csvCheckLine[7];
                                    $startDateTime  = strtotime($startDate);
                                    $year           = date('Y', $startDateTime);
                                    $month          = strtolower(strftime('%B', $startDateTime));
                                    $pdfPageNumList = explode('-', $csvCheckLine[12]);
                                    $period         = $startDate . '_' . $endDate;

                                    // agent user
                                    $agentUser = new DematpayslipUser();
                                    $agentUser->setId(0);
                                    $agentUser->setUserId('');
                                    $agentUser->setSiret($csvCheckLine[0]);
                                    $agentUser->setMatricule($csvCheckLine[8]);
                                    $agentUser->setLastname($csvCheckLine[2]);
                                    $agentUser->setFirstname($csvCheckLine[3]);
                                    $agentUser->setEmail($csvCheckLine[10]);
                                    $agentUser->setZipCity($csvCheckLine[4]);

                                    $this->_logCsvLine($this->l->t('Ligne CSV '), self::LOG_DEBUG, $agentUser, $startDate, $endDate);

                                    // process skip lines
                                    if (in_array($csvCheckLineNum, $dematpayslipProcessSkipLineIdList)) {
                                        $this->_logCsvLine('-- ' . $this->l->t('Saut de la ligne CSV : ' . ($csvCheckLineNum + 1)) . ' --', self::LOG_DEBUG, $agentUser, $startDate, $endDate);

                                        // creer le PDF des bulletins de paie a ne pas indexer
                                        if ($pdfPayslipSkip === null) {
                                            $pdfPayslipSkip = new \setasign\Fpdi\Fpdi();
                                        }
                                        // ajout des pages des bulletins de paie de l'agent a ne pas indexer
                                        $this->_logCsvLine($this->l->t('Ajout des pages PDF au bulletin de paie à ne pas indexer.'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                        foreach ($pdfPageNumList as $pdfPageNum) {
                                            $this->_logCsvLine($this->l->t('Ajout de la page ' . $pdfPageNum . '.'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                            $pdfPayslipSkip->AddPage();
                                            $pdfPayslipSkip->setSourceFile($pdfOriginFileFullPath);
                                            $tplPayslipSkipId = $pdfPayslipSkip->importPage($pdfPageNum);
                                            $pdfPayslipSkip->useTemplate($tplPayslipSkipId);
                                        }

                                        $csvCheckLineNum++;
                                        continue;
                                    }

                                    // no user but archiving for 5 years "BulletinsNoIndex/HORODATE/SIRET-MATRICULE/PERIODE.pdf"
                                    if (empty($agentUser->getEmail())) {
                                        $this->_logCsvLine('-- ' . $this->l->t('Création du PDF non dématérialisé de l\'agent (aucun email) : ' . ($csvCheckLineNum + 1)) . ' --', self::LOG_DEBUG, $agentUser, $startDate, $endDate);

                                        // check if not already archived
                                        $dematpayslipArchive = $this->_archiveCheckBeforeSave($dematpayslipArchiveMapper, $agentUser, $startDate, $endDate);

                                        if (!empty($dematpayslipArchive)) {
                                            // create PDF payslip
                                            $payslipNoDematHorodateDirPath = $this->_payslipNoIndexDir . DIRECTORY_SEPARATOR . $phaseStartDateTime->format('Y-m-d_H-i-s');
                                            $isDir = Filesystem::is_dir($payslipNoDematHorodateDirPath);
                                            if (!$isDir) {
                                                $dirCreated = Filesystem::mkdir($payslipNoDematHorodateDirPath);
                                                if (!$dirCreated) {
                                                    $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier "' . $payslipNoDematHorodateDirPath . '".');
                                                }
                                            }
                                            $payslipNoDematHorodateAgentDirPath = $payslipNoDematHorodateDirPath . DIRECTORY_SEPARATOR . $agentUser->getSiret() . '-' . $agentUser->getMatricule();
                                            $isDir = Filesystem::is_dir($payslipNoDematHorodateAgentDirPath);
                                            if (!$isDir) {
                                                $dirCreated = Filesystem::mkdir($payslipNoDematHorodateAgentDirPath);
                                                if (!$dirCreated) {
                                                    $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier "' . $payslipNoDematHorodateAgentDirPath . '".');
                                                }
                                            }
                                            $payslipNoDematPDFPath = $payslipNoDematHorodateAgentDirPath . DIRECTORY_SEPARATOR . $startDate . '_' . $endDate;
                                            $pdfPayslipNoDemat = new \setasign\Fpdi\Fpdi();

                                            // add all agent pdf pages
                                            $this->_logCsvLine($this->l->t('Ajout des pages PDF au bulletin de paie à ne pas indexer.'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                            foreach ($pdfPageNumList as $pdfPageNum) {
                                                $this->_logCsvLine($this->l->t('Ajout de la page ' . $pdfPageNum . '.'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                                $pdfPayslipNoDemat->AddPage();
                                                $pdfPayslipNoDemat->setSourceFile($pdfOriginFileFullPath);
                                                $pdfPayslipNoDematId = $pdfPayslipNoDemat->importPage($pdfPageNum);
                                                $pdfPayslipNoDemat->useTemplate($pdfPayslipNoDematId);
                                            }

                                            // create PDF payslip in no index folder
                                            $this->_logCsvLine($this->l->t('Créer le PDF du bulletin de paie non dematérialisé.'), self::LOG_DEBUG);
                                            $pdfPayslipNoDemat->Output('F', $userHomeFilesDirPath . DIRECTORY_SEPARATOR . $payslipNoDematPDFPath);
                                            $pdfPayslipFilePath = $payslipNoDematPDFPath . '.pdf';
                                            $renamed = Filesystem::rename($payslipNoDematPDFPath, $pdfPayslipFilePath);
                                            if (!$renamed) {
                                                $errorMsg = $this->l->t('Impossible de déplacer le PDF du bulletin de paie non dematérialisé "' . $payslipNoDematPDFPath . '".');
                                                $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                                $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                            }

                                            // save archive
                                            $this->_archiveSaveForAgentAndPeriod($dematpayslipArchiveMapper, $dematpayslipArchive, $pdfPayslipFilePath, $agentUser, $startDate, $endDate, $year, $month, $editionDate, $phaseStartDateTime->getTimestamp());
                                        }

                                        $csvCheckLineNum++;
                                        continue;
                                    }

                                    // get agent from email
                                    $this->_logCsvLine($this->l->t('Récupération de l\'utilisateur Nextcloud à partir de son email [' . $agentUser->getEmail() . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                    $userFindByEmailList = $userManager->getByEmail($agentUser->getEmail());

                                    // /!\ remove all users not available (FIX for multiple users with SAML)
                                    if (self::USER_REMOVE_UNKNOWN == 1) {
                                        $userFindByEmailList = self::_removeUnknownUser($userFindByEmailList);
                                    }

                                    if (count($userFindByEmailList) != 1) {
                                        $errorMsg = $this->l->t('Impossible de trouver l\'unique utilisateur Nextcloud dont l\'email est "' . $agentUser->getEmail() . '" [matricule="' . $agentUser->getMatricule() . '"].');
                                        $this->_logCsvLine($errorMsg, self::LOG_ERROR, $agentUser, $startDate, $endDate);
                                        $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                    }
                                    $userFindByEmail = current($userFindByEmailList);
                                    $userFindByEmailId = $userFindByEmail->getUID();

                                    if ($this->_process->getHasError() <= 0) {
                                        // dossier "SIRET-MATRICULE"
                                        $siretMatriculeDirName = $agentUser->getSiret() . '-' . $agentUser->getMatricule();
                                        $this->_logCsvLine($this->l->t('Vérification l\'existance du dossier matricule [' . $siretMatriculeDirName . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                        $payslipsMatriculeDirPath = $payslipsDirPath . DIRECTORY_SEPARATOR . $siretMatriculeDirName;
                                        $isDir = Filesystem::is_dir($payslipsMatriculeDirPath);
                                        if (!$isDir) {
                                            $this->_logCsvLine($this->l->t('Création du dossier matricule [' . $siretMatriculeDirName . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                            $dirCreated = Filesystem::mkdir($payslipsMatriculeDirPath);
                                            if (!$dirCreated) {
                                                $errorMsg = $this->l->t('Impossible de créer le dossier "' . $payslipsMatriculeDirPath . '" [matricule="' . $agentUser->getMatricule() . '"].');
                                                $this->_logCsvLine($errorMsg, self::LOG_ERROR, $agentUser, $startDate, $endDate);
                                                $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                            }

                                            // inserer l'utilisateur de la fiche de paie (association avec son siret, matricule et email)
                                            $this->_logCsvLine($this->l->t('Insertion de l\'utilisateur de la fiche de paie : association dossier [' . $siretMatriculeDirName . '] avec email [' . $agentUser->getEmail() . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                            $dematpayslipUserList = $this->_dematpayslipUserMapper->findAllBySiretAndMatricule($agentUser->getSiret(), $agentUser->getMatricule());
                                            if (empty($dematpayslipUserList)) {
                                                $dematpayslipUser = new DematpayslipUser();
                                                $dematpayslipUser->setUserId($userFindByEmailId);
                                                $dematpayslipUser->setSiret($agentUser->getSiret());
                                                $dematpayslipUser->setMatricule($agentUser->getMatricule());
                                                $dematpayslipUser->setEmail($agentUser->getEmail());
                                                $dematpayslipUser->setLastname($agentUser->getLastname());
                                                $dematpayslipUser->setFirstname($agentUser->getFirstname());
                                                $dematpayslipUser->setZipCity($agentUser->getZipCity());
                                                $this->_dematpayslipUserMapper->insert($dematpayslipUser);
                                            } else {
                                                $errorMsg = $this->l->t('L\'utilisateur "' . $userFindByEmailId . '" est déjà associé à un dossier SIRET-MATRICULE [siret="' . $agentUser->getSiret() . '", matricule="' . $agentUser->getMatricule() . '", email="' . $agentUser->getEmail() . '"].');
                                                $this->_logCsvLine($errorMsg, self::LOG_ERROR, $agentUser, $startDate, $endDate);
                                                $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                            }
                                        }

                                        if ($this->_process->getHasError() <= 0) {
                                            // recuperer l'utilisateur associe au dossier "SIRET-MATRICULE" cree
                                            $this->_logCsvLine($this->l->t('Récupération de l\'utilisateur associé au dossier SIRET-MATRICULE [' . $siretMatriculeDirName . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                            $dematpayslipUserList = $this->_dematpayslipUserMapper->findAllBySiretAndMatricule($agentUser->getSiret(), $agentUser->getMatricule());
                                            if (count($dematpayslipUserList) === 1) {
                                                $dematpayslipUser = $dematpayslipUserList[0];

                                                // update user for new version
                                                if (empty($dematpayslipUser->getLastname()) || empty($dematpayslipUser->getFirstname()) || empty($dematpayslipUser->getZipCity())) {
                                                    $dematpayslipUser->setLastname($agentUser->getLastname());
                                                    $dematpayslipUser->setFirstname($agentUser->getFirstname());
                                                    $dematpayslipUser->setZipCity($agentUser->getZipCity());
                                                    $this->_dematpayslipUserMapper->update($dematpayslipUser);
                                                }

                                                // set agent user id
                                                $agentUser->setId($dematpayslipUser->getId());
                                                $agentUser->setUserId($dematpayslipUser->getUserId());

                                                // si l'utilisateur n'a pas desactive la dematerialisation de son bulletin de paie
                                                if ($dematpayslipUser->getDisabled() != 1) {
                                                    // creer le partage du dossier "SIRET-MATRICULE"
                                                    $payslipsMatriculeDirPathNode = $userFolder->get($payslipsMatriculeDirPath);

                                                    // verifier si le dossier n'est pas deja partage
                                                    if (!in_array($payslipsMatriculeDirPathNode->getId(), $payslipMatriculeShareNodeIdList)) {
                                                        // partager le dossier MATRICULE en lecture seule avec l'utilisateur trouve a partir de son email
                                                        $this->_logCsvLine($this->l->t('Partage du dossier SIRET-MATRICULE [' . $siretMatriculeDirName . '] en lecture seule.'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                                        $payslipMatriculeShare = $shareManager->newShare();
                                                        $payslipMatriculeShare->setNode($payslipsMatriculeDirPathNode);
                                                        $payslipMatriculeShare->setShareType(\OC\Share\Share::SHARE_TYPE_USER);
                                                        $payslipMatriculeShare->setSharedWith($userFindByEmailId);
                                                        $payslipMatriculeShare->setPermissions(\OCP\Constants::PERMISSION_READ);
                                                        $payslipMatriculeShare->setSharedBy($this->userId);
                                                        $payslipMatriculeShare = $shareManager->createShare($payslipMatriculeShare);
                                                    } else {
                                                        $this->_logCsvLine($this->l->t('Dossier SIRET-MATRICULE [' . $siretMatriculeDirName . '] est déjà partagé.'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                                    }

                                                    // dossier "SIRET-MATRICULE/ANNEE"
                                                    $this->_logCsvLine($this->l->t('Vérification de l\'existance du dossier de l\'annéee [' . $year . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                                    $payslipsMatriculeYearDirPath = $payslipsMatriculeDirPath . DIRECTORY_SEPARATOR . $year;
                                                    $isDir = Filesystem::is_dir($payslipsMatriculeYearDirPath);
                                                    if (!$isDir) {
                                                        $this->_logCsvLine($this->l->t('Création du dossier de l\'annéee [' . $year . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                                        $dirCreated = Filesystem::mkdir($payslipsMatriculeYearDirPath);
                                                        if (!$dirCreated) {
                                                            $errorMsg = $this->l->t('Impossible de créer le dossier "' . $payslipsMatriculeYearDirPath . '[siret="' . $agentUser->getSiret() . '", matricule="' . $agentUser->getMatricule() . '", année="' . $year . '"].');
                                                            $this->_logCsvLine($errorMsg, self::LOG_ERROR, $agentUser, $startDate, $endDate);
                                                            $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                                        }
                                                    }

                                                    if ($this->_process->getHasError() <= 0) {
                                                        // check if not already archived
                                                        $dematpayslipArchive = $this->_archiveCheckBeforeSave($dematpayslipArchiveMapper, $agentUser, $startDate, $endDate);

                                                        if (!empty($dematpayslipArchive)) {
                                                            // deplacer le fichier PDF du bulletin de paie de l'agent dans "SIRET-MATRICULE/ANNEE/PERIODE.pdf"
                                                            $pdfPayslipCheckFileName = ($csvCheckLineNum + 1) . '_' . $agentUser->getMatricule() . '.pdf';
                                                            $pdfPayslipCheckFilePath = $this->_checkDirUserPath . DIRECTORY_SEPARATOR . $pdfPayslipCheckFileName;
                                                            $pdfPayslipFilePath = $payslipsMatriculeYearDirPath . DIRECTORY_SEPARATOR . $period . '.pdf';
                                                            $this->_logCsvLine($this->l->t('Déplacement du fichier PDF du bulletin de paie de la période [' . $period . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                                            $renamed = Filesystem::rename($pdfPayslipCheckFilePath, $pdfPayslipFilePath);
                                                            if (!$renamed) {
                                                                $errorMsg = $this->l->t('Impossible de déplacer le bulletin de paie "' . $pdfPayslipCheckFileName . '" dans le dossier "' . $pdfPayslipFilePath . '".');
                                                                $this->_logCsvLine($errorMsg, self::LOG_ERROR, $agentUser, $startDate, $endDate);
                                                                $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                                            }

                                                            // save archive
                                                            $this->_archiveSaveForAgentAndPeriod($dematpayslipArchiveMapper, $dematpayslipArchive, $pdfPayslipFilePath, $agentUser, $startDate, $endDate, $year, $month, $editionDate, $phaseStartDateTime->getTimestamp());
                                                        }
                                                    }
                                                } else {
                                                    // agent disabled payslip
                                                    $this->_logCsvLine($this->l->t('L\'utlisateur a désactivé la dématérialisation des fiches de paie [email=' . $agentUser->getEmail() . '].'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);

                                                    // check if not already archived
                                                    $dematpayslipArchive = $this->_archiveCheckBeforeSave($dematpayslipArchiveMapper, $agentUser, $startDate, $endDate);

                                                    if (!empty($dematpayslipArchive)) {
                                                        // create PDF payslip
                                                        $payslipNoDematHorodateDirPath = $this->_payslipNoIndexDir . DIRECTORY_SEPARATOR . $phaseStartDateTime->format('Y-m-d_H-i-s');
                                                        $isDir = Filesystem::is_dir($payslipNoDematHorodateDirPath);
                                                        if (!$isDir) {
                                                            $dirCreated = Filesystem::mkdir($payslipNoDematHorodateDirPath);
                                                            if (!$dirCreated) {
                                                                $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier "' . $payslipNoDematHorodateDirPath . '".');
                                                            }
                                                        }
                                                        $payslipNoDematHorodateAgentDirPath = $payslipNoDematHorodateDirPath . DIRECTORY_SEPARATOR . $agentUser->getSiret() . '-' . $agentUser->getMatricule();
                                                        $isDir = Filesystem::is_dir($payslipNoDematHorodateAgentDirPath);
                                                        if (!$isDir) {
                                                            $dirCreated = Filesystem::mkdir($payslipNoDematHorodateAgentDirPath);
                                                            if (!$dirCreated) {
                                                                $this->_msgList['error'][] = $this->l->t('Impossible de créeer le dossier "' . $payslipNoDematHorodateAgentDirPath . '".');
                                                            }
                                                        }
                                                        $payslipNoDematPDFPath = $payslipNoDematHorodateAgentDirPath . DIRECTORY_SEPARATOR . $startDate . '_' . $endDate;
                                                        $pdfPayslipNoDemat = new \setasign\Fpdi\Fpdi();

                                                        // add all agent pdf pages
                                                        $this->_logCsvLine($this->l->t('Ajout des pages PDF au bulletin de paie à ne pas indexer.'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                                        foreach ($pdfPageNumList as $pdfPageNum) {
                                                            $this->_logCsvLine($this->l->t('Ajout de la page ' . $pdfPageNum . '.'), self::LOG_DEBUG, $agentUser, $startDate, $endDate);
                                                            $pdfPayslipNoDemat->AddPage();
                                                            $pdfPayslipNoDemat->setSourceFile($pdfOriginFileFullPath);
                                                            $pdfPayslipNoDematId = $pdfPayslipNoDemat->importPage($pdfPageNum);
                                                            $pdfPayslipNoDemat->useTemplate($pdfPayslipNoDematId);
                                                        }

                                                        // create PDF payslip in no index folder
                                                        $this->_logCsvLine($this->l->t('Créer le PDF du bulletin de paie non dematérialisé.'), self::LOG_DEBUG);
                                                        $pdfPayslipNoDemat->Output('F', $userHomeFilesDirPath . DIRECTORY_SEPARATOR . $payslipNoDematPDFPath);
                                                        $pdfPayslipFilePath = $payslipNoDematPDFPath . '.pdf';
                                                        $renamed = Filesystem::rename($payslipNoDematPDFPath, $pdfPayslipFilePath);
                                                        if (!$renamed) {
                                                            $errorMsg = $this->l->t('Impossible de déplacer le PDF du bulletin de paie non dematérialisé "' . $payslipNoDematPDFPath . '".');
                                                            $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                                            $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                                        }

                                                        // save archive
                                                        $this->_archiveSaveForAgentAndPeriod($dematpayslipArchiveMapper, $dematpayslipArchive, $pdfPayslipFilePath, $agentUser, $startDate, $endDate, $year, $month, $editionDate, $phaseStartDateTime->getTimestamp());
                                                    }
                                                }
                                            } else {
                                                $errorMsg = $this->l->t('L\'utilisateur "' . $userFindByEmailId . '" n\'a pas de dossier SIRET-MATRICULE associé [siret="' . $agentUser->getSiret() . '", matricule="' . $agentUser->getMatricule() . '", email="' . $agentUser->getEmail() . '"].');
                                                $this->_logCsvLine($errorMsg, self::LOG_ERROR, $agentUser, $startDate, $endDate);
                                                $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                            }
                                        }
                                    }
                                }

                                if ($this->_process->getHasError() > 0) {
                                    $this->_logCsvLine('-- ' . $this->l->t('Fin de ligne CSV : ' . ($csvCheckLineNum + 1)) . ' --', self::LOG_ERROR);
                                    break;
                                } else {
                                    $this->_logCsvLine('-- ' . $this->l->t('Fin de ligne CSV : ' . ($csvCheckLineNum + 1)) . ' --', self::LOG_SUCCESS);
                                }

                                $csvCheckLineNum++;
                            }
                            fclose($csvCheckFile);

                            // creer le PDF des bulletins de paie a ne pas indexer
                            if ($pdfPayslipSkip !== null) {
                                $this->_logCsvLine($this->l->t('Créer le PDF des bulletins de paie a ne pas indexer.'), self::LOG_DEBUG);
                                $pdfPayslipSkip->Output('F', $userHomeFilesDirPath . DIRECTORY_SEPARATOR . $payslipSkipPDFPath);
                                $renamed = Filesystem::rename($payslipSkipPDFPath, $payslipSkipPDFPath . '.pdf');
                                if (!$renamed) {
                                    $errorMsg = $this->l->t('Impossible de déplacer le PDF des bulletins de paie a ne pas indexer "' . $payslipSkipPDFPath . '".');
                                    $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                    $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $csvCheckLineNum);
                                }
                            }
                        }
                    }

                    if ($this->_process->getHasError() <= 0) {
                        $phaseStep = 'report';
                        $phaseStepNum = 0;
                        $retryPhaseStepNum = $phaseStepNum;
                        if ($retry > 0) {
                            if ($this->_process->getPhaseStep() == $phaseStep) {
                                $this->_process->setPhaseStep('');
                            }
                            $retryPhaseStepNum = $this->_process->getPhaseStepNum();
                        }
                        if (empty($this->_process->getPhaseStep())) {
                            $reportSubDirName = $phaseStartDateTime->format('Y-m-d_H-i-s');
                            $reportSubDirPath = $this->_reportDirUserPath . DIRECTORY_SEPARATOR . $reportSubDirName;

                            // creer le sous dossier du rapport
                            if ($phaseStepNum >= $retryPhaseStepNum) {
                                $this->_logCsvLine($this->l->t('Création du dossier de rapport.'), self::LOG_DEBUG);
                                $dirCreated = Filesystem::mkdir($reportSubDirPath);
                                if (!$dirCreated) {
                                    $errorMsg = $this->l->t('Impossible de créer le dossier du rapport "' . $reportSubDirName . '".');
                                    $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                    $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $phaseStepNum);
                                }
                            }

                            if ($this->_process->getHasError() <= 0) {
                                // deplacer le fichier CSV Upload dans le dossier Report
                                $phaseStepNum++;
                                if ($phaseStepNum >= $retryPhaseStepNum) {
                                    $csvUploadFileName = $this->_process->getCsvOrigin();
                                    if (!empty($csvUploadFileName)) {
                                        $this->_logCsvLine($this->l->t('Déplacement du fichier CSV d\'upload dans le dossier du rapport.'), self::LOG_DEBUG);
                                        $csvUploadFileUserPath = $this->_uploadDirUserPath . DIRECTORY_SEPARATOR . $csvUploadFileName;
                                        $reportFileUserPath = $reportSubDirPath . DIRECTORY_SEPARATOR . $csvUploadFileName;
                                        $renamed = Filesystem::rename($csvUploadFileUserPath, $reportFileUserPath);
                                        if (!$renamed) {
                                            $errorMsg = $this->l->t('Impossible de déplacer le fichier CSV d\'upload "' . $csvUploadFileName . '" dans le dossier du rapport "' . $reportSubDirName . '".');
                                            $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                            $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $phaseStepNum);
                                        }
                                    }
                                }

                                // deplacer le fichier PDF Upload dans le dossier Report
                                $phaseStepNum++;
                                if ($phaseStepNum >= $retryPhaseStepNum) {
                                    $pdfUploadFileName = $this->_process->getPdfOrigin();
                                    if (!empty($pdfUploadFileName)) {
                                        $this->_logCsvLine($this->l->t('Déplacement du fichier PDF d\'upload dans le dossier du rapport.'), self::LOG_DEBUG);
                                        $pdfUploadFileUserPath = $this->_uploadDirUserPath . DIRECTORY_SEPARATOR . $pdfUploadFileName;
                                        $reportFileUserPath = $reportSubDirPath . DIRECTORY_SEPARATOR . $pdfUploadFileName;
                                        $renamed = Filesystem::rename($pdfUploadFileUserPath, $reportFileUserPath);
                                        if (!$renamed) {
                                            $errorMsg = $this->l->t('Impossible de déplacer le fichier PDF d\'upload "' . $pdfUploadFileName . '" dans le dossier du rapport "' . $reportSubDirName . '".');
                                            $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                            $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $phaseStepNum);
                                        }
                                    }
                                }

                                // deplacer le fichier CSV de traitement dans le dossier Report
                                $phaseStepNum++;
                                if ($phaseStepNum >= $retryPhaseStepNum) {
                                    if (!empty($csvCheckFileName)) {
                                        $this->_logCsvLine($this->l->t('Déplacement du fichier CSV créé après vérification dans le dossier du rapport.'), self::LOG_DEBUG);
                                        $csvCheckFileUserPath = $this->_checkDirUserPath . DIRECTORY_SEPARATOR . $csvCheckFileName;
                                        $reportFileUserPath = $reportSubDirPath . DIRECTORY_SEPARATOR . $csvCheckFileName;
                                        $renamed = Filesystem::rename($csvCheckFileUserPath, $reportFileUserPath);
                                        if (!$renamed) {
                                            $errorMsg = $this->l->t('Impossible de déplacer le fichier CSV de vérification "' . $csvCheckFileName . '" dans le dossier du rapport "' . $reportSubDirName . '".');
                                            $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                            $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $phaseStepNum);
                                        }
                                    }
                                }

                                // deplacer le fichier de log cree lors de la phase de verification
                                $phaseStepNum++;
                                if ($phaseStepNum >= $retryPhaseStepNum) {
                                    // recuperation du fichier de log cree lors de la phase de verification
                                    $logCheckFileName = self::PHASE_ID_CHECK . '_log.csv';
                                    if (!empty($logCheckFileName)) {
                                        $this->_logCsvLine($this->l->t('Déplacement du fichier de Log créé après vérification dans le dossier du rapport.'), self::LOG_DEBUG);
                                        $logCheckFileUserPath = $this->_checkDirUserPath . DIRECTORY_SEPARATOR . $logCheckFileName;
                                        $reportFileUserPath = $reportSubDirPath . DIRECTORY_SEPARATOR . $logCheckFileName;
                                        $renamed = Filesystem::rename($logCheckFileUserPath, $reportFileUserPath);
                                        if (!$renamed) {
                                            $errorMsg = $this->l->t('Impossible de déplacer le fichier de Log de vérification "' . $logCheckFileName . '" dans le dossier du rapport "' . $reportSubDirName . '".');
                                            $this->_logCsvLine($errorMsg, self::LOG_ERROR);
                                            $this->_processErrorMsg(-1, $errorMsg, $phaseStep, $phaseStepNum);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }

                if ($this->_process->getHasError() <= 0) {
                    $successMsg = 'Opération "' . self::_getPhaseName($this->_process->getPhase()) . '" terminée avec succès.';
                    $this->_logCsvLine($this->l->t($successMsg), self::LOG_SUCCESS);
                    $this->_msgList['success'][] = $successMsg;
                    $this->_processInit();
                }

                $this->_logCsvLine('## ' . $this->l->t('Fin du processus d\'indexation.') . ' ##', self::LOG_DEBUG);
                fclose($this->_logFile);
                // rename (sync file)
                $renamed = Filesystem::rename($logFilePath, $logFilePath . '.csv');
                $this->_process->setProcessing(0);
                $this->_dematpayslipProcessMapper->update($this->_process);
                // vider les lignes du process a sauter
                $this->_dematpayslipProcessSkipMapper->deleteAllByIdProcess($this->_process->getId());
            }

            return new TemplateResponse('dematpayslip', 'index', $this->_assignParamsDemat());
        } else {
            return $this->shares();
        }
    }

    /**
     * Test page
     * @NoAdminRequired
     * @NoCSRFRequired
     *
     * @return  TemplateResponse
     * @throws  \OC\User\NoUserException
     */
    public function userList() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $userEmail = filter_input(INPUT_POST, 'user_search_email', FILTER_SANITIZE_EMAIL);
            $searchList = array(
              'user_search_email' => $userEmail
            );

            if (!empty($userEmail)) {
                $userManager = \OC::$server->getUserManager();
                // /!\ remove all users not available (FIX for multiple users with SAML)
                if (self::USER_REMOVE_UNKNOWN == 1) {
                    $this->_userList = self::_removeUnknownUser($userManager->getByEmail($userEmail));
                }
            }

            return new TemplateResponse('dematpayslip', 'user_list', $this->_assignUserList($searchList));
        } else {
            return $this->shares();
        }
    }

    /**
     * Liste des agents
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     *
     * @return  TemplateResponse
     * @throws  \OC\User\NoUserException
     */
    public function agentList() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematFolders();

            return new TemplateResponse('dematpayslip', 'agent_list', $this->_assignAgentList());
        } else {
            return $this->shares();
        }
    }

    /**
     * Exporter la liste des agents au format CSV
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     *
     * @return  TemplateResponse
     * @throws  \OC\User\NoUserException
     */
    public function agentCsv() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematFolders();

            if (count($this->_msgList['error'])<=0) {
                $dematpayslipUserList = $this->_dematpayslipUserMapper->findAll(array(array('disabled', 'desc'), array('siret'), array('matricule')));

                if (!empty($dematpayslipUserList)) {
                    $dateTimeNow = new \DateTime('now', \OC::$server->getDateTimeZone()->getTimeZone());
                    $csvFilePath = $this->_exportDirUserPath . DIRECTORY_SEPARATOR .  $dateTimeNow->format('Y-m-d_H-i-s') . '_agent';
                    $csvFile = Filesystem::fopen($csvFilePath, 'w+');

                    if (!$csvFile) {
                        $errorMsg = $this->l->t('Ouverture du fichier CSV.');
                        $this->_msgList['error'][] = $errorMsg;
                    }

                    if (count($this->_msgList['error']) <= 0) {
                        $csvDelimiter = $this->_csvDelimiter;
                        $csvEnclosure = $this->_csvEnclosure;

                        $csvFields   = array();
                        $csvFields[] = $this->l->t('Utilisateur');
                        $csvFields[] = $this->l->t('Siret');
                        $csvFields[] = $this->l->t('Matricule');
                        $csvFields[] = $this->l->t('Email');
                        $csvFields[] = $this->l->t('Papier ?');

                        $res = fputcsv($csvFile, $csvFields, $csvDelimiter, $csvEnclosure);
                        if (!$res) {
                            $errorMsg = $this->l->t('Ecriture des colonnes dans le fichier CSV.');
                            $this->_msgList['error'][] = $errorMsg;
                        }

                        if (count($this->_msgList['error']) <= 0) {
                            foreach ($dematpayslipUserList as $dematpayslipUser) {
                                $agent = \OC::$server->getUserManager()->get($dematpayslipUser->getUserId());

                                $csvFields = array();
                                //$csvFields[] = $dematpayslipUser->getUserId();
                                $csvFields[] = $agent->getDisplayName();
                                $csvFields[] = $dematpayslipUser->getSiret();
                                $csvFields[] = $dematpayslipUser->getMatricule();
                                $csvFields[] = $dematpayslipUser->getEmail();
                                $csvFields[] = $dematpayslipUser->getDisabled();

                                $res = fputcsv($csvFile, $csvFields, $csvDelimiter, $csvEnclosure);
                                if (!$res) {
                                    $errorMsg = $this->l->t('Ecriture de la ligne dans le fichier CSV.');
                                    $this->_msgList['error'][] = $errorMsg;
                                    break;
                                }
                            }
                        }

                        $res = fclose($csvFile);
                        if (!$res) {
                            $errorMsg = $this->l->t('Fermeture du fichier CSV.');
                            $this->_msgList['error'][] = $errorMsg;
                        }

                        if (count($this->_msgList['error']) <= 0) {
                            // rename (sync files)
                            $renamed = Filesystem::rename($csvFilePath, $csvFilePath . '.csv');
                            $this->_msgList['success'][] = $this->l->t('La liste des agents a bien été exporté au format CSV.');
                        }
                    }
                }
            }

            return new TemplateResponse('dematpayslip', 'agent_list', $this->_assignAgentList());
        } else {
            return $this->shares();
        }
    }

    /**
     * Archive list
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     *
     * @return  TemplateResponse
     * @throws  \OC\User\NoUserException
     */
    public function archiveList() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematFolders();

            $archived = filter_input(INPUT_GET, 'archived', FILTER_SANITIZE_NUMBER_INT);
            if ($archived === null) {
                $archived = -1;
            }

            return new TemplateResponse('dematpayslip', 'archive_list', $this->_assignArchiveList($archived));
        } else {
            return $this->shares();
        }
    }

    /**
     * Archive action
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     *
     * @return  TemplateResponse
     * @throws  \OC\User\NoUserException
     */
    public function archiveAction() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematFolders();

            $archived = filter_input(INPUT_POST, 'archived', FILTER_SANITIZE_NUMBER_INT);
            if ($archived === null) {
                $archived = -1;
            }
            $refresh = false;

            $action = filter_input(INPUT_POST, 'action', FILTER_SANITIZE_STRING);
            $archiveSelectIdList = filter_input(INPUT_POST, 'archive_select', FILTER_DEFAULT, FILTER_REQUIRE_ARRAY);

            if ($action == 'refresh_status') {
                // refresh all archive status
                $refresh = true;
            } elseif ($action == 'create') {
                $dematpayslipArchiveMapper = new DematpayslipArchiveMapper(\OC::$server->getDatabaseConnection());

                if (!empty($archiveSelectIdList)) {
                    foreach ($archiveSelectIdList as $archiveSelectId) {
                        $dematpayslipArchive = $dematpayslipArchiveMapper->find(intval($archiveSelectId));
                        if (!empty($dematpayslipArchive)) {
                            $archiveFlowNum = 1; // archiving for 5 years by default

                            // get dematpayslip user
                            if (!empty($dematpayslipArchive->getDematpayslipIdUser())) {
                                $dematpayslipUser = $this->_dematpayslipUserMapper->find($dematpayslipArchive->getDematpayslipIdUser());
                                if ($dematpayslipUser->getDisabled() == 0) {
                                    $archiveFlowNum = 2; // archiving for 50 years
                                }
                            }

                            // Pastell API config
                            $pastellConfigFlowId = $this->config->getAppValue($this->appName, 'pastell_config_flow' . $archiveFlowNum . '_id', '');
                            $pastellConfigFlowMapper = new PastellConfigFlowMapper(\OC::$server->getDatabaseConnection());
                            $pastellConfigFlow = $pastellConfigFlowMapper->find($pastellConfigFlowId);

                            try {
                                // Pastell API connection
                                $pastellAPI = new PastellAPI($pastellConfigFlow->getIdConfig());

                                $canCreateArchive = false;
                                if (empty($dematpayslipArchive->getPastellIdDocument())) {
                                    $canCreateArchive = true;
                                } else {
                                    // get and modify document last action
                                    $responseData = $pastellAPI->getEntityDocument($pastellConfigFlow->getIdE(), $dematpayslipArchive->getPastellIdDocument());
                                    if (!empty($responseData['last_action']['action'])) {
                                        $dematpayslipArchive->setPastellLastAction($responseData['last_action']['action']);
                                        $dematpayslipArchiveMapper->update($dematpayslipArchive);
                                    }
                                }

                                if ($canCreateArchive === false) {
                                    $this->_msgList['error'][] = $this->l->t('L\'archive "' . $dematpayslipArchive->getFilePath() . '" a déjà été créée.');
                                } else {
                                    // create document
                                    $responseData = $pastellAPI->postEntityDocument($pastellConfigFlow->getIdE(), array(
                                        'type' => 'pdf-generique'
                                    ));
                                    $dematpayslipArchive->setPastellIdDocument($responseData['id_d']);
                                    $dematpayslipArchive->setPastellLastAction($responseData['last_action']['action']);
                                    $dematpayslipArchiveMapper->update($dematpayslipArchive);

                                    if ($dematpayslipArchive->getPastellLastAction() == 'creation') {
                                        // set document title
                                        $responseData = $pastellAPI->patchEntityDocument($pastellConfigFlow->getIdE(), $dematpayslipArchive->getPastellIdDocument(), array(
                                            'libelle' => $dematpayslipArchive->getId() . '_' . $dematpayslipArchive->getStartDate() . '_' . $dematpayslipArchive->getEndDate()
                                        ));
                                        $dematpayslipArchive->setPastellLastAction($responseData['content']['last_action']['action']);
                                        $dematpayslipArchiveMapper->update($dematpayslipArchive);
                                    }
                                    if ($dematpayslipArchive->getPastellLastAction() == 'modification') {
                                        // send document file
                                        $fileId = $dematpayslipArchive->getIdFile();
                                        $filePath = Filesystem::getPath($fileId);
                                        $file = Filesystem::getFileInfo($filePath);
                                        $fileName = $file->getName();
                                        $fileContent = 'TEST';
                                        //$fileContent = Filesystem::file_get_contents($filePath);
                                        $responseData = $pastellAPI->postEntityDocumentFile($pastellConfigFlow->getIdE(), $dematpayslipArchive->getPastellIdDocument(), 'document', 0, $fileName, $fileContent);
                                        $dematpayslipArchive->setPastellLastAction($responseData['content']['last_action']['action']);
                                        $dematpayslipArchiveMapper->update($dematpayslipArchive);

                                        // set orientation action on document
                                        $responseData = $pastellAPI->postEntityDocumentAction($pastellConfigFlow->getIdE(), $dematpayslipArchive->getPastellIdDocument(), 'orientation');
                                        if ($responseData['result'] == true) {
                                            // get and modify document last action
                                            $responseData = $pastellAPI->getEntityDocument($pastellConfigFlow->getIdE(), $dematpayslipArchive->getPastellIdDocument());
                                            if (!empty($responseData['last_action']['action'])) {
                                                $dematpayslipArchive->setPastellLastAction($responseData['last_action']['action']);
                                                $dematpayslipArchiveMapper->update($dematpayslipArchive);
                                            }
                                        }
                                    }
                                    if ($dematpayslipArchive->getPastellLastAction() == 'preparation-suppression-automatique') {
                                        // create "sae_config" JSON file and send it
                                        $saeConfigArr = array(
                                            'matricule' => $dematpayslipArchive->getMatricule(),
                                            'mois' => $dematpayslipArchive->getMonth(),
                                            'annee' => $dematpayslipArchive->getYear(),
                                            'nom' => $dematpayslipArchive->getLastname(),
                                            'prenom' => $dematpayslipArchive->getFirstname(),
                                            'date_edition' => $dematpayslipArchive->getEditionDate(),
                                            'periode_2_de_paie' => $dematpayslipArchive->getEndDate(),
                                            'periode_1_de_paie' => $dematpayslipArchive->getStartDate(),
                                        );
                                        $jsonFileName = 'sae_config.json';
                                        $jsonFilePath = $this->_tmpDirUserPath . DIRECTORY_SEPARATOR . $jsonFileName;
                                        Filesystem::fopen($jsonFilePath, 'w+');
                                        Filesystem::file_put_contents($jsonFilePath, json_encode($saeConfigArr));

                                        // add meta-data on document
                                        $jsonFileContent = Filesystem::file_get_contents($jsonFilePath);
                                        $responseData = $pastellAPI->postEntityDocumentFile($pastellConfigFlow->getIdE(), $dematpayslipArchive->getPastellIdDocument(), 'sae_config', 0, $jsonFileName, $jsonFileContent);
                                        $dematpayslipArchive->setPastellLastAction($responseData['content']['last_action']['action']);
                                        $dematpayslipArchiveMapper->update($dematpayslipArchive);
                                        $this->_msgList['success'][] = $this->l->t('L\'archive "' . $dematpayslipArchive->getFilePath() . '" a été créée.');
                                    }
                                }
                            } catch (\Exception $e) {
                                $this->_msgList['error'][] = $e->getMessage();
                            }
                        }
                    }
                }
            } elseif ($action == 'delete') {
                $dematpayslipArchiveMapper = new DematpayslipArchiveMapper(\OC::$server->getDatabaseConnection());

                if (!empty($archiveSelectIdList)) {
                    foreach ($archiveSelectIdList as $archiveSelectId) {
                        $dematpayslipArchive = $dematpayslipArchiveMapper->find(intval($archiveSelectId));
                        if (!empty($dematpayslipArchive)) {
                            if (!empty($dematpayslipArchive->getPastellIdDocument())) {
                                $this->_msgList['error'][] = $this->l->t('Impossible de supprimer le bulletin de paie "' . $dematpayslipArchive->getFilePath() . '" en cours d\'archivage.');;
                            } else {
                                $dematpayslipArchiveMapper->delete($dematpayslipArchive);
                            }
                        }
                    }
                }
            }

            return new TemplateResponse('dematpayslip', 'archive_list', $this->_assignArchiveList($archived, $refresh));
        } else {
            return $this->shares();
        }
    }

    /**
     * Archive CSV
     *
     * @NoAdminRequired
     * @NoCSRFRequired
     *
     * @return TemplateResponse
     * @throws \Exception
     */
    public function archiveCsv() {
        // verifier l'utilisateur
        if ($this->userId === $this->_dematpayslipConfigList['user_id']) {
            $this->_initDematFolders();

            $dematpayslipArchiveMapper = new DematpayslipArchiveMapper(\OC::$server->getDatabaseConnection());
            $dematpayslipArchiveList = $dematpayslipArchiveMapper->findAllArchived(-1);

            if (!empty($dematpayslipArchiveList)) {
                $dateTimeNow = new \DateTime('now', \OC::$server->getDateTimeZone()->getTimeZone());
                $csvFilePath = $this->_exportDirUserPath . DIRECTORY_SEPARATOR . $dateTimeNow->format('Y-m-d_H-i-s') . '_archive';
                $csvFile = Filesystem::fopen($csvFilePath, 'w+');

                if (!$csvFile) {
                    $errorMsg = $this->l->t('Ouverture du fichier CSV.');
                    $this->_msgList['error'][] = $errorMsg;
                }

                if (count($this->_msgList['error']) <= 0) {
                    $csvDelimiter = $this->_csvDelimiter;
                    $csvEnclosure = $this->_csvEnclosure;

                    $csvFields = array();
                    $csvFields[] = $this->l->t('Chemin');
                    $csvFields[] = $this->l->t('Siret');
                    $csvFields[] = $this->l->t('Matricule');
                    $csvFields[] = $this->l->t('Nom');
                    $csvFields[] = $this->l->t('Prénom');
                    $csvFields[] = $this->l->t('Début');
                    $csvFields[] = $this->l->t('Fin');
                    $csvFields[] = $this->l->t('Edition');
                    $csvFields[] = $this->l->t('Archive');
                    $csvFields[] = $this->l->t('Dernière action');

                    $res = fputcsv($csvFile, $csvFields, $csvDelimiter, $csvEnclosure);
                    if (!$res) {
                        $errorMsg = $this->l->t('Ecriture des colonnes dans le fichier CSV.');
                        $this->_msgList['error'][] = $errorMsg;
                    }

                    if (count($this->_msgList['error']) <= 0) {
                        foreach ($dematpayslipArchiveList as $dematpayslipArchive) {
                            $csvFields = array();
                            $csvFields[] = $dematpayslipArchive->getFilePath();
                            $csvFields[] = $dematpayslipArchive->getSiret();
                            $csvFields[] = $dematpayslipArchive->getMatricule();
                            $csvFields[] = $dematpayslipArchive->getLastname();
                            $csvFields[] = $dematpayslipArchive->getFirstname();
                            $csvFields[] = $dematpayslipArchive->getStartDate();
                            $csvFields[] = $dematpayslipArchive->getEndDate();
                            $csvFields[] = $dematpayslipArchive->getEditionDate();
                            $csvFields[] = $dematpayslipArchive->getPastellIdDocument();
                            $csvFields[] = $dematpayslipArchive->getPastellLastAction();

                            $res = fputcsv($csvFile, $csvFields, $csvDelimiter, $csvEnclosure);
                            if (!$res) {
                                $errorMsg = $this->l->t('Ecriture de la ligne dans le fichier CSV.');
                                $this->_msgList['error'][] = $errorMsg;
                                break;
                            }
                        }
                    }

                    $res = fclose($csvFile);
                    if (!$res) {
                        $errorMsg = $this->l->t('Fermeture du fichier CSV.');
                        $this->_msgList['error'][] = $errorMsg;
                    }

                    if (count($this->_msgList['error']) <= 0) {
                        // rename (sync files)
                        $renamed = Filesystem::rename($csvFilePath, $csvFilePath . '.csv');
                        $this->_msgList['success'][] = $this->l->t('La liste des archives a bien été exporté au format CSV.');
                    }
                }
            }

            return new TemplateResponse('dematpayslip', 'archive_list', $this->_assignArchiveList());
        } else {
            return $this->shares();
        }
    }

    /**
     * Liste des bulletins de paie
     *
     * @return TemplateResponse
     */
    public function shares() {
        $this->_dematpayslipUserList = $this->_dematpayslipUserMapper->findAllByUserId($this->userId);
        $nbDematpayslipUserList = count($this->_dematpayslipUserList);
        if ($nbDematpayslipUserList >= 1) {
//            foreach ($this->_dematpayslipUserList as $dematpayslipUser) {
//                if ($dematpayslipUser->getDisabled()==1) {
//                    $msgWarning =  $this->l->t('Vous avez désactivé la dématérialisation des fiches de paie');
//                    if ($nbDematpayslipUserList > 1) {
//                        $msgWarning .= ' (SIRET : ' . $dematpayslipUser->getSiret() . ')';
//                    }
//                    $this->_msgList['warning'][] = $msgWarning;
//                }
//            }
        } else {
            $this->_msgList['error'][] = $this->l->t('Cet utilisateur n\'a pas encore de bulletin de paie dématérialisé.');
        }

        return new TemplateResponse('dematpayslip', 'shares', $this->_assignParamsShares());
    }
}
